from __future__ import absolute_import
from __future__ import unicode_literals

import requests
from laipvt.interface.middlewareinterface import MiddlewareInterface
from laipvt.handler.middlewarehandler import IdentityConfigHandler
from laipvt.handler.confighandler import CheckResultHandler
from laipvt.sysutil.template import FileTemplate
from laipvt.sysutil.util import path_join, log, status_me, walk_sql_path
from laipvt.sysutil.conf import YamlConfig
from laipvt.sysutil.gvalue import LAIPVT_BASE_DIR
from laipvt.model.sql import SqlModule
from laipvt.sysutil.command import CREATE_DB
from laipvt.sysutil.util import get_yaml_config


class IdentityController(MiddlewareInterface):
    def __init__(self, result: CheckResultHandler, handler: IdentityConfigHandler, template: str):
        super(IdentityController, self).__init__(result, handler, template)
        self.identity_conf_tmp = path_join("/tmp", "appsettings.json")
        self.identity_conf_template = path_join(self.template, "config.tmpl")
        self.identity_conf_file = path_join(self.base_dir, "config/appsettings.json")
        self.identity_nginx_tmp = path_join("/tmp", "nginx-identity.conf")
        self.identity_nginx_template = path_join(self.template, "nginx-identity.tmpl")
        self.identity_sql_template = path_join(self.template, "identity_sqls")
        self.log_path = path_join(self.base_dir, "logs")
        self.identity_sql_file = path_join(self.base_dir)
        self.identity = IdentityConfigHandler()
        self.cfg = YamlConfig(path_join(LAIPVT_BASE_DIR, "middleware"), suffix="yaml").read_dir()
        self.cfg["identity"] = self.identity.get_config_with_check_result()["identity"]
        self.cfg["identity"]["userstore"] = "Express"
        self.cfg["identity"]["ipaddress"] = self.handler.cfg["ipaddress"]
        # self.cfg["redis"]["ipaddress"] = self.handler.cfg["ipaddress"]
        self.user = self.cfg["mysql"]["username"]
        self.password = self.cfg["mysql"]["password"]
        self.port = int(self.cfg["mysql"]["port"])
        self.cfg.update(self.check_result.__dict__)
    def _generic_config(self):
        log.info("渲染 identity 配置文件")
        if self.cfg["mysql"]["is_deploy"]:
            self.cfg["mysql"]["ipaddress"] = self.handler.cfg["ipaddress"]
        if self.cfg["redis"]["is_deploy"]:
            self.cfg["redis"]["ipaddress"] = self.handler.cfg["ipaddress"]
        self.cfg['is_standalone'] = False
        if len(self.cfg["mysql"]["ipaddress"]) == 1:
            self.cfg['is_standalone'] = True
        for num_id in range(len(self.master_server)):
            self.cfg["localhost"] = self.master_server[num_id].ipaddress
            FileTemplate(self.cfg, self.identity_conf_template, self.identity_conf_tmp).fill()
            self.send_config_file(self.master_server[num_id], self.identity_conf_tmp, self.identity_conf_file)
            self.send_config_file(self.master_server[num_id], self.identity_sql_template, self.identity_sql_file)
        self.generate_docker_compose_file(self.cfg)

    def _proxy_on_nginx(self):
        log.info("渲染 nginx identity 代理配置文件")
        FileTemplate(self.cfg, self.identity_nginx_template, self.identity_nginx_tmp).fill()
        self.update_nginx_config()

    def _check(self):
        super().wait_for_service_start()
        for i in range(len(self.master_server)):
            try:
                log.info("检查 {IP}:{PORT} 上的 Identity 健康性...".format(IP=self.master_server[i].ipaddress, PORT=self.cfg["identity"]["nginx_proxy_port"]))
                requests.get(
                    "http://{IP}:{PORT}".format(
                        IP=self.master_server[i].ipaddress, PORT=self.cfg["identity"]["nginx_proxy_port"]
                    )
                )
                log.info("Identity检查通过")
            except Exception as e:
                log.error(e)
                log.error("{IP} 上的 Identity 服务异常".format(IP=self.master_server[i].ipaddress))
                exit(2)

    def init_mysql(self):
        if not self.cfg["mysql"]["is_deploy"]:
            self.master_host = self.cfg["mysql"]["ipaddress"][0]
        else:
            self.cfg["mysql"]["ipaddress"] = self.handler.cfg["ipaddress"][0]
            self.master_host = self.cfg["mysql"]["ipaddress"]
        log.info("初始化mysql identity数据表")
        db_info = walk_sql_path(path_join(self.template, "identity_sqls"))
        sql = SqlModule(host=self.master_host, port=self.port, user=self.user, passwd=self.password)
        for db_name, sql_files in db_info.items():
            create_db = CREATE_DB.format(db_name=db_name)
            sql.insert_sql(create_db)
            sql.use_db(db_name)
            for sql_file in sql_files:
                sql.import_from_file(sql_file, file_eof=";\n")
        self.create_logs_dir(self.log_path)

    @status_me("middleware")
    def deploy_identity(self):
        self._generic_config()
        self._proxy_on_nginx()
        self.send_docker_compose_file()
        self.init_mysql()
        self.start()
        self._check()

    @status_me("middleware")
    def update_identity_config(self):
        self.cfg["identity"]["userstore"] = "Commander"
        self._generic_config()
        self.send_docker_compose_file()
        self.restart()

    def deploy(self):
        self.deploy_identity()



from __future__ import absolute_import
from __future__ import unicode_literals

from laipvt.sysutil.util import to_object

class ListHandler(object):
    def __init__(self, l: list):
        self.l = l
        self.iter = iter(l)

    def check(self, value: str) -> bool:
        return value in self.l

    def get(self) -> list:
        return self.l

    def next(self):
        try:
            return next(self.iter)
        except StopIteration:
            return False

    def length(self) -> int:
        return len(self.l)

class DictHandler(object):
    def __init__(self, d: dict):
        self.d = to_object(d)

    def get(self) -> dict:
        return self.d

    def length(self) -> int:
        return len(self.d)


from __future__ import absolute_import
from __future__ import unicode_literals

from laipvt.interface.kubeinterface import KubeInterface
from laipvt.handler.confighandler import CheckResultHandler
from laipvt.handler.packagehandler import KubePackageHandler


class KubeController(KubeInterface):
    def __init__(self, result: CheckResultHandler, kube: KubePackageHandler.parse, *args, **kwargs):
        super(KubeController, self).__init__(result, kube, *args, **kwargs)



from __future__ import absolute_import
from __future__ import unicode_literals

from redis.sentinel import Sentinel
from laipvt.interface.middlewareinterface import MiddlewareInterface
from laipvt.handler.middlewarehandler import RedisConfigHandler
from laipvt.handler.confighandler import CheckResultHandler
from laipvt.sysutil.template import FileTemplate
from laipvt.sysutil.util import path_join, log, status_me


class RedisController(MiddlewareInterface):
    def __init__(self, result: CheckResultHandler, handler: RedisConfigHandler, template: str):
        super(RedisController, self).__init__(result, handler, template)
        self.redis_conf_tmp = path_join("/tmp", "redis.conf")
        self.redis_conf_template = path_join(self.template, "config.tmpl")
        self.redis_conf_file = path_join(self.base_dir, "conf", "redis.conf")
        self.redis_stand_tmp = path_join("/tmp", "redis-sentinel.conf")
        self.redis_stand_template = path_join(self.template, "config-sentinel.tmpl")
        self.redis_stand_file = path_join(self.base_dir, "conf", "redis-sentinel.conf")
        self.redis_cfg = RedisConfigHandler().get_config_with_check_result()
        self.redis_cfg["redis"]["ipaddress"] =  self.handler.cfg["ipaddress"]

    def _generic_config(self):
        log.info("渲染 Redis 配置文件")
        for num_id in range(len(self.master_server)):
            if num_id == 0:
                self.redis_cfg["redis"]["master_ip"] = self.master_server[num_id].ipaddress
                self.redis_cfg["redis"]["isslave"] = False
            else:
                self.redis_cfg["redis"]["isslave"] = True
            FileTemplate(self.redis_cfg, self.redis_conf_template, self.redis_conf_tmp).fill()
            FileTemplate(self.redis_cfg, self.redis_stand_template, self.redis_stand_tmp).fill()
            self.send_config_file(self.master_server[num_id], self.redis_conf_tmp, self.redis_conf_file)
            self.send_config_file(self.master_server[num_id], self.redis_stand_tmp, self.redis_stand_file)
            self.generate_docker_compose_file(self.redis_cfg)

    def _check(self) -> bool:
        super().wait_for_service_start()
        pool = []
        for host in self.master_server:
            redis_endpoint = (host.ipaddress, self.redis_cfg["redis"]["port_sentinel"])
            pool.append(redis_endpoint)
        try:
            key = "__XX_LAIYE_TEST"
            value = "XXXVALUEXXX"
            cli = Sentinel(pool)
            master = cli.master_for(
                self.redis_cfg["redis"]["master_name"],
                password=self.redis_cfg["redis"]["password"],
                db=15
                )
            if master:
                log.info("Redis master 服务正常")
            if self.handler.cfg["deploy_type"] == "single":
                return True
            r = master.set(key, value)
            if r:
                log.info("Redis master 写入正常")
                slave = cli.slave_for(
                    self.redis_cfg["redis"]["master_name"],
                    password=self.redis_cfg["redis"]["password"],
                    db=15
                )
                if slave.get(key).decode() == value:
                    log.info("Redis slave 读取正常")
                    return True
        except Exception as e:
            log.error("Redis 集群配置失败,请检查：{}".format(e))

    @status_me("middleware")
    def deploy_redis(self):
        if self.check_is_deploy(self.redis_cfg):
            self._generic_config()
            self.send_docker_compose_file()
            self.start()
            self._check()

    def deploy(self):
        self.deploy_redis()



from __future__ import absolute_import
from __future__ import unicode_literals

import json
import requests
from requests.adapters import HTTPAdapter
from requests.auth import HTTPBasicAuth
from laipvt.interface.middlewareinterface import MiddlewareInterface
from laipvt.handler.confighandler import CheckResultHandler
from laipvt.handler.middlewarehandler import HarborConfigHandler
from laipvt.sysutil.util import path_join, ssh_obj, log, status_me, post, get, put
from laipvt.sysutil.template import FileTemplate
from laipvt.model.cmd import ComposeModel
from laipvt.sysutil.command import BASH_RUN, TARGZ_DECOMPRESSION, CHMOD_USER_GROUP


class HarborController(MiddlewareInterface):
    def __init__(self, result: CheckResultHandler, handler: HarborConfigHandler, template: str):
        super(HarborController, self).__init__(result, handler, template)

        self.harbor_conf_tmp = path_join("/tmp", "harbor.yml")
        self.harbor_conf_template = path_join(self.template, "config.tmpl")
        self.harbor_conf_file = path_join(self.base_dir, "harbor.yml")
        self.harbor_service_template = path_join(self.template, "harbor.service.tmpl")
        self.harbor_service_tmp = path_join("/tmp", "harbor.service")
        self.harbor_service_file = "/usr/lib/systemd/system/harbor.service"
        self.habor_data = path_join(self.template, "data")
        self._docker_compose_file = path_join(self.base_dir, "docker-compose.yaml")
        self.harbor_cfg = HarborConfigHandler().get_config_with_check_result()
        self.harbor_cfg["harbor"]["log_location"] = path_join(self.base_dir, "var/log/harbor")
        self.harbor_cfg["harbor"]["data_volume"] = path_join(self.base_dir, "data")
        self.harbor_cfg["harbor"]["deploy_path"] = self.base_dir
        self.harbor_cfg["harbor"]["ipaddress"] = [ip.ipaddress for ip in self.harbor_server]
        self.harbor_nginx_tmp = path_join("/tmp", "nginx-harbor.conf")
        self.harbor_nginx_template = path_join(self.template, "nginx-harbor.tmpl")
        self.harbor_nginx_file = path_join(self.deploy_dir, "nginx/http/nginx-harbor.conf")
        self.master_harbor = self.harbor_server[0]

    def _generic_config(self):
        """
        如果是多机Harbor 只将data目录拷贝到第一台harbor上，传输第二台的时候将data.tar.gz过滤掉，使用内部同步机制，节约部署时间
        :return:
        """
        log.info("渲染分发 Harbor 安装包及配置文件")
        for num_id, server in enumerate(self.harbor_server):
            self.harbor_cfg["harbor"]["localhost_ip"] = server.ipaddress
            FileTemplate(self.harbor_cfg, self.harbor_conf_template, self.harbor_conf_tmp).fill()
            FileTemplate(self.harbor_cfg, self.harbor_service_template, self.harbor_service_tmp).fill()
            if num_id == 0:
                self.send_config_file(server=server, src=self.template, dest=self.base_dir)
            else:
                self.send_config_file(server=server, src=self.template, dest=self.base_dir, ignore='data.tar.gz')
            self.send_config_file(server=server, src=self.harbor_conf_tmp, dest=self.harbor_conf_file)
            self.send_config_file(server=server, src=self.harbor_service_tmp, dest=self.harbor_service_file)

    def ssh_cli(self, cmd):
        for server in self.harbor_server:
            ssh_cli = ssh_obj(ip=server.ipaddress, user=server.username, password=server.password, port=server.port)
            res = ssh_cli.run_cmd(cmd)
            ssh_cli.close()
        return res

    def _install_harbor(self):
        log.info("安装harbor")
        cmd = BASH_RUN.format(path_join(self.base_dir, "install.sh"))
        res = self.ssh_cli(cmd)
        if res["code"] != 0:
            log.error("{} {}".format(res["stdout"], res["stderr"]))
            exit(2)

    def _unpack_data(self):
        """
        只解压第一台Harbor的数据目录
        :return:
        """
        ssh_cli = ssh_obj(
            ip=self.master_harbor.ipaddress,
            user=self.master_harbor.username,
            password=self.master_harbor.password,
            port=self.master_harbor.port
        )
        cmd = [
            TARGZ_DECOMPRESSION.format(path_join(self.base_dir, "data.tar.gz"), self.base_dir),
            CHMOD_USER_GROUP.format(
                "999", "999",
                path_join(self.base_dir, "data", "database"),
                path_join(self.base_dir, "data", "redis")),
            CHMOD_USER_GROUP.format(
                "10000", "10000",
                path_join(self.base_dir, "data", "secret"),
                path_join(self.base_dir, "data", "registry"))
        ]
        res_list = ssh_cli.run_cmdlist(cmd)
        for res in res_list:
            if res["code"] != 0:
                log.error("Harbor Data数据解压或授权失败:{} {}".format(res["stdout"], res["stderr"]))
                exit(2)

    def _start(self):
        compose_cmd = ComposeModel(self.docker_compose_file)
        for server in self.harbor_server:
            log.info("启动 %s %s服务" % (server.ipaddress, self.middleware_name))
            ssh_cli = ssh_obj(ip=server.ipaddress, user=server.username, password=server.password, port=server.port)
            try:
                res = ssh_cli.run_cmd(compose_cmd.up())
                # for res in self.ssh_cli(compose_cmd.up()):
                if res["code"] != 0:
                    log.error("启动 %s: %s服务失败: %s %s" % (
                        server.ipaddress, self.middleware_name,
                        res["stdout"], res["stderr"]
                        ))
                    exit(2)
            except Exception as e:
                log.error(e)
                exit(2)
            finally:
                ssh_cli.close()

    def _check(self):
        super().wait_for_service_start()
        s = requests.Session()
        s.mount('http://', HTTPAdapter(max_retries=3))
        s.mount('https://', HTTPAdapter(max_retries=3))
        try:
            for server in self.harbor_server:
                url = "http://{IP}:{PORT}/api/v2.0/health".format(
                    IP=server.ipaddress, PORT=self.harbor_cfg["harbor"]["http_port"]
                )
                log.info("检查 {}:{} 上的 Harbor 健康性".format(server.ipaddress, self.harbor_cfg["harbor"]["http_port"]))
                result = s.get(url, timeout=5).json()
                if result["status"] == "healthy":
                    log.info("{} Harbor 服务运行正常".format(server.ipaddress))
                else:
                    log.error("{IP} 上的 Harbor 服务异常".format(IP=server.ipaddress))
                    log.error("错误日志: {}".format(result))
                    exit(2)
        except Exception as e:
            log.error(e)
            exit(2)

    def _change_password(self):
        """
        只修改第一台Harbor的密码即可，第二台使用Habor配置文件中的密码，所以无需修改
        :return:
        """
        log.info("修改 Harbor 登陆密码")
        try:
            # for server in self.harbor_server:
            url = "http://{IP}:{PORT}/api/v2.0/users/1/password".format(
                IP=self.master_harbor.ipaddress, PORT=self.harbor_cfg["harbor"]["http_port"])
            headers = {"Content-Type": "application/json"}
            data_dit = {"old_password": "Harbor12345", "new_password": "{}".format(self.harbor_cfg["harbor"]["password"])}
            result = requests.put(url=url, json=data_dit, headers=headers, auth=HTTPBasicAuth('admin', 'Harbor12345'))
            if result:
                log.info("{} Harbor 密码修改成功".format(self.master_harbor.ipaddress))
            else:
                log.error("{} Harbor 密码修改失败,新密码至少包含1个大写字母、1个小写字母和1个数字".format(self.master_harbor.ipaddress))
                log.error(result)
                exit(2)
        except Exception as e:
            log.error(e)
            log.error("{} Harbor 密码修改失败,请检查Harbor服务是否正常启动或端口是否可达".format(self.master_harbor.ipaddress))
            exit(2)

    def registries(self):
        """
        两台Harbor互相配置仓库管理
        :return:
        """
        for num_id, ip in enumerate(self.harbor_server):
            if num_id == 0:
                harbor_ipaddress = self.harbor_server[1].ipaddress
                # url = "http://{}:{}/api/v2.0/registries".format(self.harbor_server[1].ipaddress, self.harbor_cfg["harbor"]["harbor_http_port"])
            else:
                harbor_ipaddress = self.harbor_server[0].ipaddress
                # url = "http://{}:{}/api/v2.0/registries".format(self.harbor_server[0].ipaddress, self.harbor_cfg["harbor"]["harbor_http_port"])
            url = "http://{}:{}/api/v2.0/registries".format(harbor_ipaddress,
                                                            self.harbor_cfg["harbor"]["harbor_http_port"])
            data = {
                "credential": {
                    "access_key": "admin",
                    "access_secret": "{}".format(self.harbor_cfg["harbor"]["password"]),
                    "type": "basic"
                },
                "name": "Harbor_slave",
                "url": "http://{}:{}".format(self.harbor_server[num_id].ipaddress,
                                             self.harbor_cfg["harbor"]["harbor_http_port"]),
                "insecure": False,
                "id": 100,
                "type": "harbor"
            }
            res = self.interface_res(url, data)
            if res.status_code != 201:
                log.error(
                    "{} Harbor 仓库管理配置失败, 状态码{} ".format(harbor_ipaddress, res.status_code))
                exit(2)
            else:
                log.info("{} Harbor 仓库管理配置成功 ".format(harbor_ipaddress))

    def interface_res(self, url, data):
        try:
            res = post(url, data=json.dumps(data), timeout=5,
                       auth=('admin', '{}'.format(self.harbor_cfg["harbor"]["password"])))
            return res
        except Exception as e:
            log.error(e)
            exit(2)

    def policies(self, model, policy_id=''):
        """
        配置Harbor复制管理，首次调用需要设置手动同步模式，后面手动调用手动出发同步接口，手动同步完毕后，在将模式改为触发同步
        :return:
        """
        for num_id, server in enumerate(self.harbor_server):
            if num_id == 0:
                harbor_ipaddress = self.harbor_server[1].ipaddress
            else:
                harbor_ipaddress = self.harbor_server[0].ipaddress
            url = "http://{}:{}/api/v2.0/replication/policies{}".format(harbor_ipaddress,
                                                                        self.harbor_cfg["harbor"]["harbor_http_port"], policy_id)
            data = {
                "name": "replication_to_salve1",
                "dest_registry": {
                    "id": 100,
                    "name": "Harbor_slave",
                    "type": "harbor",
                    "url": "http://{}:{}".format(self.harbor_server[num_id].ipaddress,
                                                 self.harbor_cfg["harbor"]["harbor_http_port"]),
                    "credential": {
                        "type": "basic",
                        "access_key": "admin",
                        "access_secret": "{}".format(self.harbor_cfg["harbor"]["password"])
                    },
                    "insecure": False,
                },
                "dest_namespace": "",
                        "trigger": {
                            "type": model,
                            "trigger_settings": {
                                "cron": ""
                            }
                        },
                        "id": 100,
                        "enabled": True,
                        "deletion": False,
                        "override": True,
                    }
            if policy_id == '':
                res = post(url, json.dumps(data), timeout=5,
                           auth=('admin', '{}'.format(self.harbor_cfg["harbor"]["password"])))
            else:
                res = put(url, json.dumps(data), timeout=5,
                          auth=('admin', '{}'.format(self.harbor_cfg["harbor"]["password"])))
            if res.status_code == 200 or res.status_code == 201:
                log.info("Harbor 复制管理配置成功 ")
            else:
                log.error(
                    "Harbor 复制管理配置失败, 状态码{} ".format(res.status_code))
                exit(2)

    def get_policies_id(self):
        """
        获取复制管理的唯一ID
        :return:
        """
        url = "http://{}:{}/api/v2.0/replication/policies?name=replication_to_salve1"\
            .format(self.master_harbor.ipaddress,
                    self.harbor_cfg["harbor"]["harbor_http_port"])
        try:
            res = get(url, timeout=5,
                      auth=('admin', '{}'.format(self.harbor_cfg["harbor"]["password"]))).json()[0]["id"]
            return res
        except Exception as e:
            log.error(e)

    def manual_sync(self):
        """
        触发手动同步
        :return:
        """
        data = {
            "policy_id": self.get_policies_id()
        }
        url = "http://{}:{}/api/v2.0/replication/executions"\
            .format(self.master_harbor.ipaddress,
                    self.harbor_cfg["harbor"]["harbor_http_port"])
        res = self.interface_res(url, data)
        if res.status_code != 201:
            log.error("Harbor 同步镜像失败, 状态码{} ".format(res.status_code))
            exit(2)
        else:
            log.info("Harbor镜像同步成功")

    def _proxy_on_nginx(self):
        log.info("渲染 Nginx 代理配置文件")
        FileTemplate(self.harbor_cfg, self.harbor_nginx_template, self.harbor_nginx_tmp).fill()
        self.update_nginx_config()
        # self.generate_docker_compose_file(self.harbor_cfg)

    @status_me("basesystem")
    def install_harbor(self):
        self._generic_config()
        self._install_harbor()
        self._unpack_data()
        self._start()
        self._check()
        self._change_password()
        self._proxy_on_nginx()
        if len(self.harbor_server) > 1:
            self.registries()
            self.policies("manual")
            self.manual_sync()
            self.policies("event_based", '/100')





from __future__ import absolute_import
from __future__ import unicode_literals

import requests
from laipvt.interface.middlewareinterface import MiddlewareInterface
from laipvt.handler.confighandler import CheckResultHandler
from laipvt.handler.middlewarehandler import RabbitmqConfigHandler
from laipvt.sysutil.util import ssh_obj, log, path_join, status_me
from laipvt.sysutil.template import FileTemplate
from laipvt.model.server import ServerModel


class RabbitmqController(MiddlewareInterface):
    def __init__(self, result: CheckResultHandler, handler: RabbitmqConfigHandler, template: str):
        super(RabbitmqController, self).__init__(result, handler, template)
        self.rabbitmq_cfg = RabbitmqConfigHandler().get_config_with_check_result()
        self.rabbitmq_conf_tmp = path_join("/tmp", "rabbitmq.config")
        self.rabbitmq_conf_template = path_join(self.template, "config.tmpl")
        self.rabbitmq_conf_file = path_join(self.base_dir, "config/rabbitmq.conf")
        if self.rabbitmq_cfg["rabbitmq"]["is_deploy"]:
            self.rabbitmq_cfg["rabbitmq"]["ipaddress"] = self.handler.cfg["ipaddress"]
        self.rabbitmq_service_file_template = path_join(self.template, "rabbitmq_service.tmpl")
        self.rabbitmq_service_file_dest = path_join("/tmp", "rabbitmq_service.yaml")
        self.rabbitmq_service_file_remote = path_join(self.base_dir, "svc", "rabbitmq_service.yaml")

    def _generic_config(self):
        log.info("渲染 Rabbitmq 配置文件")
        if len(self.master_server) == 1:
            log.info("使用单机模式部署")
            self.rabbitmq_cfg["is_standalone"] = True

        elif len(self.master_server) == 3:
            log.info("使用集群模式部署")
            self.rabbitmq_cfg["is_standalone"] = False
        else:
            log.info("集群模式需要3台主机,请检查前置检查配置是否正确")
            exit()

        for num_id in range(len(self.master_server)):
            self.rabbitmq_cfg["rabbitmq"]["NODENAME"] = "rabbit@saas-rabbitmq-0%s" % (num_id + 1)
            self.rabbitmq_cfg["rabbitmq"]["HOSTNAME"] = "saas-rabbitmq-0%s" % (num_id + 1)
            FileTemplate(self.rabbitmq_cfg, self.rabbitmq_conf_template, self.rabbitmq_conf_tmp).fill()
            self.generate_docker_compose_file(self.rabbitmq_cfg)
            self._send_docker_compose_file(self.master_server[num_id])
            self.send_config_file(self.master_server[num_id], self.rabbitmq_conf_tmp, self.rabbitmq_conf_file)


    def _send_docker_compose_file(self, host):
        log.info("分发%s docker-compose配置文件至 %s:%s" % (self.middleware_name, host.ipaddress, self.docker_compose_file))
        ssh_cli = ssh_obj(ip=host.ipaddress, user=host.username, password=host.password, port=host.port)
        try:
            ssh_cli.put(self.docker_compose_file_tmp, self.docker_compose_file)
        except Exception as e:
            log.error(e)
            exit(2)
        finally:
            ssh_cli.close()

    def init_rabbitmq_cluster(self):
        log.info("设置 Rabbitmq policy")
        flag = False
        for server in self.master_server[1:]:
            ssh_cli = ssh_obj(ip=server.ipaddress, user=server.username, password=server.password, port=server.port)
            results = ssh_cli.run_cmd("docker exec rabbitmq rabbitmqctl stop_app")
            log.info("stop_app: {} {}".format(results["stdout"], results["stderr"]))
            results = ssh_cli.run_cmd("docker exec rabbitmq rabbitmqctl reset")
            log.info("reset: {} {}".format(results["stdout"], results["stderr"]))
            results = ssh_cli.run_cmd("docker exec rabbitmq rabbitmqctl join_cluster rabbit@saas-rabbitmq-01")
            log.info("join_cluster: {} {}".format(results["stdout"], results["stderr"]))
            results = ssh_cli.run_cmd("docker exec rabbitmq rabbitmqctl start_app")
            log.info("start_app: {} {}".format(results["stdout"], results["stderr"]))

            if flag:
                set_policy_cmd = """docker exec rabbitmq rabbitmqctl set_policy mirror_queue "^" '{"ha-mode":"all", "ha-sync-mode":"automatic"}' """
                results = ssh_cli.run_cmd(set_policy_cmd)
                if results["code"] != 0:
                    log.error("RabbitMQ初始化失败:{} {}".format(results["stdout"], results["stderr"]))
                    exit(2)
                else:
                    log.info("RabbitMQ初始化成功：{} {}".format(results["stdout"], results["stderr"]))
            flag = True
            ssh_cli.close()

    def _check(self):
        super().wait_for_service_start()
        for i in range(len(self.master_server)):
            try:
                log.info("检查 {IP} 上的 Rabbitmq 健康性...".format(IP=self.master_server[i].ipaddress))
                requests.get(
                    "http://{IP}:{PORT}".format(
                        IP=self.master_server[i].ipaddress, PORT=self.rabbitmq_cfg["rabbitmq"]["manage_port"]
                    )
                )
                log.info("Rabbitmq检查通过")
            except Exception as e:
                log.error(e)
                log.error("{IP} 上的 Rabbitmq 服务异常".format(IP=self.master_server[i].ipaddress))
                exit(2)
        if not self.rabbitmq_cfg["is_standalone"]:
            self.init_rabbitmq_cluster()

    def create_rabbitmq_service_kubernetes(self):
        log.info("渲染初始化RabbitMQ Service in Kubernetes")
        server = ServerModel(self.harbor_server)
        FileTemplate(self.rabbitmq_cfg, self.rabbitmq_service_file_template, self.rabbitmq_service_file_dest).fill()
        server.send_file(self.rabbitmq_service_file_dest, self.rabbitmq_service_file_remote)

        log.info("在kubernetes集群内创建RabbitMQ Service")
        cmd = "kubectl apply -f {}".format(self.rabbitmq_service_file_remote)
        ssh_cli = ssh_obj(ip=self.harbor_server[0].ipaddress, user=self.harbor_server[0].username,
                          password=self.harbor_server[0].password, port=self.harbor_server[0].port)
        results = ssh_cli.run_cmd(cmd)
        if results["code"] != 0:
            log.error("kubernetes集群内创建RabbitMQ Service失败:{} {}".format(results["stdout"], results["stderr"]))
            exit(2)

    @status_me("middleware")
    def deploy_rabbitmq(self):
        if self.check_is_deploy(self.rabbitmq_cfg):
            self._generic_config()
            self.start()
            self._check()
        self.create_rabbitmq_service_kubernetes()

    def deploy(self):
        self.deploy_rabbitmq()



from __future__ import absolute_import
from __future__ import unicode_literals
import re
import os
import pymysql
from laipvt.helper.exception import ModelError
from laipvt.sysutil.util import path_join, run_local_cmd, backup_file


class SqlModule(object):
    def __init__(self, *args, **kwargs):
        self.conn = pymysql.connect(**kwargs)
        self.mysql_cmd = "mysql -h {} -u{} -p{} -P{}".format(
            kwargs.get("host"),
            kwargs.get("user"),
            kwargs.get("passwd"),
            kwargs.get("port")
        )
        self.mysqldump_cmd = "mysqldump -h {} -u{} -p{} -P{}".format(
            kwargs.get("host"),
            kwargs.get("user"),
            kwargs.get("passwd"),
            kwargs.get("port")
        )

    def use_db(self, db_name):
        self.conn.select_db(db_name)

    def _read_file(self, file_path, eof=';'):
        try:
            with open(file_path) as f:
                ret = f.read()
                ret = re.sub(r'/\*.*?\*/', "\n", ret, flags=re.S)
                ret = re.sub(r"--.*\n", "\n", ret)
                ok = ret.split(eof)
                return ok
        except Exception as e:
            raise ModelError("SqlModule._read_file, 读取文件失败，文件路径: {}, 错误信息: {}".format(file_path, e)) from None

    def _read_file_lines(self, file_path, eof=";"):
        try:
            with open(file_path) as f:
                content = f.readlines()
                result = []
                sql = ""
                for line in content:
                    if not line.strip():
                        continue
                    sql += line.strip()
                    if line.strip().find(eof) >= 0:
                        ret = re.sub(r'/\*.*?\*/', "\n", sql, flags=re.S)
                        ret = re.sub(r"--.*\n", "\n", ret)
                        result.append(ret)
                        sql = ""
                return result
        except Exception as e:
            raise ModelError("SqlModule._read_file, 读取文件失败，文件路径: {}, 错误信息: {}".format(file_path, e)) from None

    def import_from_file(self, path, file_eof=";"):
        cursor = self.conn.cursor(cursor=pymysql.cursors.DictCursor)
        for sql_request in self._read_file(path, file_eof):
            if sql_request.strip():
                try:
                    cursor.execute(sql_request)
                except Exception as e:
                    try:
                        new_sql = sql_request.split(file_eof)
                        for s in new_sql:
                            if len(s) > 0:
                                try:
                                    cursor.execute(s)
                                except Exception as e:
                                    raise e
                    except Exception as e:
                        raise ModelError("SqlModule.import_from_file, 错误信息: {}".format(e)) from None
        cursor.close()

    def import_from_file_commander(self, path, file_eof=";"):
        cursor = self.conn.cursor(cursor=pymysql.cursors.DictCursor)
        for sql_request in self._read_file_lines(path, file_eof):
            if sql_request.strip():
                try:
                    cursor.execute(sql_request)
                except Exception as e:
                    try:
                        new_sql = sql_request.split(file_eof)
                        for s in new_sql:
                            if len(s) > 0:
                                try:
                                    cursor.execute(s)
                                except Exception as e:
                                    raise e
                    except Exception as e:
                        raise ModelError("SqlModule.import_from_file, 错误信息: {}".format(e)) from None
        cursor.close()


    def insert_sql(self, sql_statements):
        cursor = self.conn.cursor(cursor=pymysql.cursors.DictCursor)
        try:
            cursor.execute(sql_statements)
            self.conn.commit()
        except Exception as e:
            raise ModelError("SqlModule.insert_sql, 错误信息: {}".format(e)) from None
        finally:
            cursor.close()


    def select(self, sql_statements):
        cursor = self.conn.cursor(cursor=pymysql.cursors.DictCursor)
        info = None
        try:
            cursor.execute(sql_statements)
            info = cursor.fetchall()
        except Exception as e:
            raise ModelError("SqlModule.select, 错误信息: {}".format(e)) from None
        finally:
            cursor.close()
            return info if info is not None else False

    def run_sql(self, sql):
        cursor = self.conn.cursor(cursor=pymysql.cursors.DictCursor)
        try:
            cursor.execute(sql)
        except Exception as e:
            raise ModelError("SqlModule.run_sql, 错误信息: {}".format(e)) from None
        finally:
            cursor.close()

    def import_from_dir(self, db, path):
        cmd = "for i in $(ls {}); do {} {} < {}/$i; done".format(path, self.mysql_cmd, db, path)
        run_local_cmd(cmd)

    def backup_db(self, db, target):
        target_file_path = path_join(target, db + ".sql")
        if os.path.isfile(target_file_path):
            backup_file(target_file_path)
        cmd = "{} {}".format(self.mysqldump_cmd, "{} > {}".format(
            db, target_file_path
        ))
        res = run_local_cmd(cmd)
        return True if res["code"] == 0 else False

    def close(self):
        self.conn.close()


from __future__ import absolute_import
from __future__ import unicode_literals

from laipvt.interface.serviceinterface import ServiceInterface
from laipvt.sysutil.util import path_join, status_me


class NlpController(ServiceInterface):
    def __init__(self, check_result, service_path):
        super(NlpController, self).__init__(check_result, service_path)
        self.project = "mage"

        self.info_engine_data_src = path_join(self.data_dir, "laiye-information-extract-engine")
        self.info_engine_data_remote = path_join(self.deploy_dir, "laiye-information-extract-engine")
        self.doc_classifer_data_src = path_join(self.data_dir, "laiye-doc-classifier")
        self.doc_classifer_data_remote = path_join(self.deploy_dir, "laiye-doc-classifier")
        self.bert_service_data_src = path_join(self.data_dir, "bert-service-tf")
        self.bert_service_data_remote = path_join(self.deploy_dir, "bert-service-tf")
        self.poi_search_engine_src = path_join(self.data_dir, "laiye-poi-search-engine")
        self.poi_search_engine_remote = path_join(self.deploy_dir, "data", "laiye-poi-search-engine")
        self.bert_service_pt_data_src = path_join(self.data_dir, "bert-service-pt")
        self.bert_service_pt_data_remote = path_join(self.deploy_dir, "bert-service-pt")

    @status_me("nlp")
    def prepare_nlp(self):
        self._send_file(src=self.info_engine_data_src, dest=self.info_engine_data_remote)
        self._send_file(src=self.doc_classifer_data_src, dest=self.doc_classifer_data_remote)
        self._send_file(src=self.bert_service_data_src, dest=self.bert_service_data_remote)
        self._send_file(src=self.bert_service_pt_data_src, dest=self.bert_service_pt_data_remote)
        self._send_file(src=self.poi_search_engine_src, dest=self.poi_search_engine_remote)

    @status_me("nlp")
    def push_nlp_images(self):
        self.push_images(self.project)

    @status_me("nlp")
    def start_nlp_service(self):
        self.start_service(project=self.project, version=self.private_deploy_version)

    def run(self):
        self.prepare_nlp()
        self.push_nlp_images()
        self.start_nlp_service()



from __future__ import absolute_import
from __future__ import unicode_literals

import os
from laipvt.interface.middlewareinterface import MiddlewareInterface
from laipvt.handler.middlewarehandler import OcrConfigHandler
from laipvt.handler.confighandler import CheckResultHandler
from laipvt.sysutil.util import path_join, log, ssh_obj
from laipvt.model.cmd import DockerImageModel, ComposeModel
from laipvt.sysutil.template import FileTemplate
from laipvt.sysutil.command import RESTART_LICSERVER


class OcrController(MiddlewareInterface):
    def __init__(self, service_path: object, result: CheckResultHandler, handler: OcrConfigHandler, template: str):
        super(OcrController, self).__init__(result, handler, template)
        self.project = "mage"
        self.service_path = service_path
        self.harbor_cfg = self.handler.get_all_config_with_check_result()
        try:
            harbor_ip = self.harbor_cfg["harbor"]["ipaddress"][0]
        except IndexError:
            harbor_ip = self.check_result.servers.get_role_ip("harbor")[0]
        self.registry_hub = "{}:{}".format(harbor_ip, self.harbor_cfg["harbor"]["http_port"])
        self.deploy_service_list = self.service_path.config.services

    def push_images(self):
        for image in os.listdir(self.service_path.images):
            image_path = path_join(self.service_path.images, image)
            log.info("将镜像push到私有仓库: {}".format(image_path))
            # print(image_path)
            docker = DockerImageModel(image=image_path, project=self.project, repo=self.registry_hub)
            docker.run()

    def start_licserver(self, renew=False, **kwargs):
        licserver = self.servers.get_role_obj("licserver")[0]
        log.info("分发授权文件文件至 %s:/etc/intsig/licServer.lic" % licserver.ipaddress )
        ssh_cli = ssh_obj(ip=licserver.ipaddress, user=licserver.username, password=licserver.password, port=licserver.port)
        try:
            if renew:
                lcs_file_path = kwargs.get("license_file")
            else:
                lcs_file_path = self.service_path.license_file

            ssh_cli.put(lcs_file_path, "/etc/intsig/licServer.lic")
            cmd = RESTART_LICSERVER
            ssh_cli.run_cmd(cmd)
        except Exception as e:
            log.error(e)
            exit(2)
        finally:
            ssh_cli.close()

    def generate_data(self, module_name: str) -> dict:
        data = {}
        data["name"] = module_name
        data["image_name"] = path_join(self.registry_hub, self.project, module_name)
        data["port"] = self.handler.get_ocr_module_port(module_name)
        log.info("获取{}的端口号为{}".format(data['name'], data['port']))
        data["machine_type"] = "gpu" if self.service_path.config.require_tfserver else "cpu"
        return data

    def generate_docker_compose_file_ocr(self, module_name: str, data_dict: dict) -> bool:
        log.info("渲染%s docker-compose配置文件" % module_name)
        compose_tmp = path_join("/tmp", "docker-compose-{}.yaml".format(module_name))
        FileTemplate(data_dict, path_join(self.template, "docker-compose.tmpl"), compose_tmp).fill()
        return compose_tmp if os.path.isfile(compose_tmp) else False

    def deploy(self, server, compose_file_local, compose_file_remote):
        compose_cmd = ComposeModel(compose_file_remote)
        ssh_cli = ssh_obj(ip=server.ipaddress, user=server.username, password=server.password,
                          port=server.port)
        try:
            log.info("发送配置文件{}到服务器{}".format(compose_file_local, server.ipaddress))
            ssh_cli.put(compose_file_local, compose_file_remote)
            log.info("启动服务{}".format(compose_cmd.up()))
            ssh_cli.run_cmd(compose_cmd.up())
        except Exception as e:
            log.error(e)
            exit(2)
        finally:
            ssh_cli.close()

    def deploy_ocr_module(self):
        server = self.servers.get_role_obj("licserver")[0]
        for module in self.deploy_service_list:
            compose_file_remote = path_join(self.check_result.deploy_dir, module, "docker-compose.yaml")
            data = self.generate_data(module)
            compose_file = self.generate_docker_compose_file_ocr(module, data)
            log.info("开始部署ocr 模型 {}".format(module))
            self.deploy(server, compose_file, compose_file_remote)

    def restart_ocr_module(self):
        server = self.servers.get_role_obj("licserver")[0]
        for module in self.deploy_service_list:
            compose_file_remote = path_join(self.check_result.deploy_dir, module, "docker-compose.yaml")
            compose_cmd = ComposeModel(compose_file_remote)
            ssh_cli = ssh_obj(ip=server.ipaddress, user=server.username,
                              password=server.password, port=server.port)
            try:
                log.info("重新启动服务{}".format(compose_cmd.restart()))
                ssh_cli.run_cmd(compose_cmd.restart())
            except Exception as e:
                log.error(e)
                exit(2)
            finally:
                ssh_cli.close()

    def renew_license(self, license_file):
        self.start_licserver(renew=True, license_file=license_file)
        self.restart_ocr_module()

    def run(self):
        self.push_images()
        self.start_licserver()
        self.deploy_ocr_module()




from __future__ import absolute_import
from __future__ import unicode_literals

from laipvt.interface.middlewareinterface import MiddlewareInterface
from laipvt.handler.middlewarehandler import KeepalivedConfigHandler
from laipvt.handler.confighandler import CheckResultHandler
from laipvt.sysutil.template import FileTemplate
from laipvt.sysutil.util import path_join, ssh_obj, log, status_me

class KeepalivedController(MiddlewareInterface):
    def __init__(self, result: CheckResultHandler, handler: KeepalivedConfigHandler, template: str):
        super(KeepalivedController, self).__init__(result, handler, template)
        self.conf_template = path_join(self.template, "keepalived.conf")
        self.template_file_name = ("keepalived_master.conf", "keepalived_slave1.conf", "keepalived_slave2.conf")
        self.keepalived_config_file = path_join(self.base_dir, "conf", "{}.conf".format(self.middleware_name))
        self.keepalived_cfg = KeepalivedConfigHandler().get_config_with_check_result()
        self.keepalived_cfg["keepalived"]["ipaddress"] = self.handler.cfg["ipaddress"]

    def _generic_config(self):
        log.info("渲染 Keepalived 配置文件")
        for num_id, server in enumerate(self.master_server):
            ssh_cli = ssh_obj(ip=server.ipaddress, user=server.username, password=server.password, port=server.port)
            cmd = "ifconfig |grep {}  -B 1|grep -Po '^.*(?=:)'".format(server.ipaddress)
            results = ssh_cli.run_cmd(cmd)
            self.keepalived_cfg["keepalived"]["network_name"] = results["stdout"].split()[0]
            src = path_join(self.template, self.template_file_name[num_id])
            dest = path_join("/tmp", self.template_file_name[num_id])
            FileTemplate(self.keepalived_cfg, src, dest).fill()
            self.send_config_file(self.master_server[num_id], dest, self.keepalived_config_file)
            self.generate_docker_compose_file(self.keepalived_cfg)

    @status_me("middleware")
    def deploy_keepalived(self):
        self._generic_config()
        self.send_docker_compose_file()
        self.start()




from __future__ import absolute_import
from __future__ import unicode_literals

from laipvt.interface.serviceinterface import ServiceInterface
from laipvt.sysutil.util import path_join


class OcrStandardController(ServiceInterface):
    def __init__(self, check_result, service_path):
        super(OcrStandardController, self).__init__(check_result, service_path)
        self.project = "mage"

        self.ocr_ctpn_model_data_src = path_join(self.data_dir, "ocr-ctpn-tf-server")
        self.ocr_ctpn_model_data_remote = path_join(self.deploy_dir, "ocr-ctpn-tf-server")
        self.ocr_text_recognition_data_src = path_join(self.data_dir, "ocr-text-recognition-tf-server")
        self.ocr_text_recognition_data_remote = path_join(self.deploy_dir, "ocr-text-recognition-tf-server")
        self.semantic_correct_src = path_join(self.data_dir, "semantic-correct")
        self.semantic_correct_remote = path_join(self.deploy_dir, "semantic-correct")
        self.unet_table_src = path_join(self.data_dir, "ocr-unet-table-tf-serving")
        self.unet_table_remote = path_join(self.deploy_dir, "ocr-unet-table-tf-serving")

    def prepare_ocr(self):
        self._send_file(src=self.ocr_ctpn_model_data_src, dest=self.ocr_ctpn_model_data_remote)
        self._send_file(src=self.ocr_text_recognition_data_src, dest=self.ocr_text_recognition_data_remote)
        self._send_file(src=self.semantic_correct_src, dest=self.semantic_correct_remote)
        self._send_file(src=self.unet_table_src, dest=self.unet_table_remote)

    def run(self):
        self.prepare_ocr()
        self.push_images(project=self.project)
        self.start_service(project=self.project, version=self.private_deploy_version)



from __future__ import absolute_import
from __future__ import unicode_literals

import os
import json
import requests
import time
from minio import Minio
from laipvt.interface.serviceinterface import ServiceInterface
from laipvt.sysutil.util import path_join, log, status_me, walk_sql_path
from laipvt.model.sql import SqlModule
from laipvt.sysutil.command import CREATE_DB, HELM_LIST, HELM_INSTALL_COMMANDER, COMMANDER_UPGRADE_INSTALL

class CommanderController(ServiceInterface):
    def __init__(self, check_result, service_path):
        super(CommanderController, self).__init__(check_result, service_path)
        self.namespaces = ["rpa", "proxy"]
        self.istio_injection_namespaces = ["rpa", "mid", ]
        self.project = "rpa"

        self.nginx_template = path_join(self.templates_dir, "nginx/http/nginx-commander.tmpl")
        self.nginx_tmp = path_join("/tmp", "nginx-commander.conf")
        self.nginx_file_remote = path_join(self.deploy_dir, "nginx/http/nginx-commander.conf")
        if self.middleware_cfg["mysql"]["is_deploy"]:
            self.mysql_host = "mysql.default.svc"
            self.mysql_port = 3306
        else:
            self.mysql_host = self.middleware_cfg["mysql"]["ipaddress"][0]
            self.mysql_port = self.middleware_cfg["mysql"]["port"]

    @status_me("commander")
    def init_commander_mysql(self):
        sql_path = self.service_path.sqls
        log.info(sql_path)
        db_info = walk_sql_path(sql_path)
        sql = SqlModule(host=self.middleware_cfg.mysql.ipaddress[0], port=int(self.middleware_cfg.mysql.port),
                        user=self.middleware_cfg.mysql.username, passwd=self.middleware_cfg.mysql.password)
        for db_name, sql_files in db_info.items():
            create_db = CREATE_DB.format(db_name=db_name)
            sql.insert_sql(create_db)
            sql.import_from_dir(db_name, path_join(sql_path, db_name))
            #sql.use_db(db_name)
            #for sql_file in sql_files:
            #    sql.import_from_file_commander(sql_file, file_eof=";")

    @status_me("commander")
    def push_commander_images(self):
        self.push_images(self.project)

    def start_service(self, project, version):
        self._send_file(src=self.service_path.charts, dest=self.service_charts_remote)
        for service, processes in self.service_path.config.services.items():
            for process in processes:
                log.info("{}开始部署".format(process))

                check_cmd = HELM_LIST.format(process, process)
                check_results = self._exec_command_to_host(cmd=check_cmd, server=self.servers[0], check_res=False)
                if check_results["code"] == 0:
                    log.warning("{} helm部署记录中已经存在，不做更新，如需要更新，可以先行删除".format(process))

                else:
                    self._create_logs_dir(service)
                    file_path = os.path.join(self.service_charts_remote, process)
                    # print(file_path)
                    if self.middleware_cfg["redis"]["is_deploy"]:
                        config_server = "\,".join(
                            [
                                "{}:{}\,serviceName={}\,allowAdmin=true".format(
                                    server,
                                    self.middleware_cfg["redis"]["port_sentinel"],
                                    self.middleware_cfg["redis"]["master_name"]
                                ) for server in self.middleware_servers.get_role_ip("master")
                            ]
                        )
                    else:
                        config_server = "{}:{}".format(self.middleware_cfg["redis"]["ipaddress"][0],
                                                       self.middleware_cfg["redis"]["port"])

                    if self.middleware_cfg["config"]["deploy_https"]:
                        oidc_authority="https://{}:{}".format(
                            self.middleware_cfg["identity"]["lb"],
                            self.middleware_cfg["identity"]["nginx_proxy_port"])
                    else:
                        oidc_authority="http://{}:{}".format(
                            self.middleware_cfg["identity"]["lb"],
                            self.middleware_cfg["identity"]["nginx_proxy_port"])

                    cmd = HELM_INSTALL_COMMANDER.format(
                        process=process, replicas=self.replicas,
                        registry_hub=path_join(self.registry_hub, project),
                        image_name=process, image_tag=version,
                        pvt_work_dir=self.deploy_dir,
                        config_server=config_server,
                        config_server_passwd=self.middleware_cfg["redis"]["password"],
                        mysql_host=self.mysql_host, mysql_port=self.mysql_port,
                        mysql_user=self.middleware_cfg["mysql"]["username"],
                        mysql_password=self.middleware_cfg["mysql"]["password"],
                        etcd_endpoint=self.etcd_endpoint, mysql_database="uibot_global",
                        mysql_charset="utf8mb4",
                        oidc_authority=oidc_authority,
                        oidc_secret="laiye",
                        file_path=file_path)

                    self._exec_command_to_host(cmd=cmd, server=self.servers[0])

    def upgrade_service(self, project):
        for service, processes in self.service_path.config.services.items():
            for process in processes:
                log.info("{}开始更新".format(process))
                self._create_logs_dir(service)
                file_path = os.path.join(self.service_path.charts, process)

                if self.middleware_cfg["redis"]["is_deploy"]:
                    config_server = "\,".join(
                        [
                            "{}:{}\,serviceName={}\,allowAdmin=true".format(
                                server,
                                self.middleware_cfg["redis"]["port_sentinel"],
                                self.middleware_cfg["redis"]["master_name"]
                            ) for server in self.middleware_servers.get_role_ip("master")
                        ]
                    )
                else:
                    config_server = "{}:{}".format(self.middleware_cfg["redis"]["ipaddress"][0],
                                                   self.middleware_cfg["redis"]["port"])

                cmd = COMMANDER_UPGRADE_INSTALL.format(
                    process=process, replicas=self.replicas,
                    registry_hub=path_join(self.registry_hub, project),
                    image_name=process, image_tag=self.private_deploy_version,
                    pvt_work_dir=self.deploy_dir,
                    config_server=config_server,
                    config_server_passwd=self.middleware_cfg["redis"]["password"],
                    mysql_host=self.mysql_host, mysql_port=self.mysql_port,
                    mysql_user=self.middleware_cfg["mysql"]["username"],
                    mysql_password=self.middleware_cfg["mysql"]["password"],
                    etcd_endpoint=self.etcd_endpoint, mysql_database="uibot_global",
                    mysql_charset="utf8mb4",
                    oidc_authority="http://{}:{}".format(self.middleware_cfg["identity"]["lb"], self.middleware_cfg["identity"]["nginx_proxy_port"]),
                    oidc_secret="laiye",
                    file_path=file_path)

                self._exec_command_to_host(cmd=cmd, server=self.servers[0])

    @status_me("commander")
    def start_commander_service(self):
        self.start_service(project=self.project, version=self.private_deploy_version)

    @status_me("commander")
    def commander_proxy_on_nginx(self):
        self.proxy_on_nginx(self.nginx_template, self.nginx_tmp, self.nginx_file_remote)

    @status_me("commander")
    def create_namespace(self):
        self._create_namespace(
            namespaces=self.namespaces,
            istio_injection_namespaces=self.istio_injection_namespaces
        )

    @status_me("commander")
    def init_minio_data(self):
        for bucket in self.service_info.buckets:
            try:
                if self.middleware_cfg.minio.is_deploy:
                    endpoint = "{}:{}".format(self.middleware_cfg.minio.lb, self.middleware_cfg.minio.nginx_proxy_port)
                else:
                    endpoint = "{}:{}".format(self.middleware_cfg.minio.ipaddress[0], self.middleware_cfg.minio.port)
                cli = Minio(
                    endpoint,
                    self.middleware_cfg.minio.username,
                    self.middleware_cfg.minio.password,
                    secure=False
                )
                if not cli.bucket_exists(bucket):
                    cli.make_bucket(bucket)
                    # policy_read_only = {
                    #     "Version": "2012-10-17",
                    #     "Statement": [
                    #         {
                    #             "Action": [
                    #                 "s3:GetObject"
                    #             ],
                    #             "Effect": "Allow",
                    #             "Principal": "*",
                    #             "Resource": [
                    #                 "arn:aws:s3:::{}/*".format(bucket)
                    #             ],
                    #             "Sid": ""
                    #         }
                    #     ]
                    # }
                    # cli.set_bucket_policy(bucket, json.dumps(policy_read_only))

            except Exception as e:
                log.error("Minio初始化数据失败:{}".format(e))
                exit(2)

    def login_tenant(self):
        log.info("登录租户管理平台")
        data = {'userName': 'admin', 'password': '123456'}
        if self.middleware_cfg["config"]["deploy_https"]:
            protocol="https"
        else:
            protocol="http"
        login_url = "{protocol}://{host}:{port}/api/global/account/webLogin".format(
            protocol=protocol, host=self.middleware_cfg.nginx.lb, port=self.middleware_cfg.nginx.commander_tenant_port
        )
        response = requests.post(login_url, json=data, verify=False)
        auth_cookie = ''
        # print(response)
        ret = response.json()
        if ret['code'] == 0:
            cookies = response.cookies
            auth_cookie = {'GlobalUser': cookies.get('GlobalUser', None)}
            # f"GlobalUser={cookies.get('GlobalUser', None)}"
            log.info(auth_cookie)
            log.info("Login succeed")
            return auth_cookie
        else:
            log.error("Login error!")
            exit(2)

    def tenant_init_mysql(self, auth_cookie):
        log.info("租户平台配置MySql数据库")
        db_mysql = {
            "name": "mysql-1",
            "host": self.mysql_host,
            "port": self.mysql_port,
            "dbName": "uibot_rpa",
            "userName": self.middleware_cfg.mysql.username,
            "password": self.middleware_cfg.mysql.password,
            "type": "10"
        }
        if self.middleware_cfg["config"]["deploy_https"]:
            protocol="https"
        else:
            protocol="http"
        mysql_url = "{protocol}://{host}:{port}/api/global/database/create".format(
            protocol=protocol, host=self.middleware_cfg.nginx.lb, port=self.middleware_cfg.nginx.commander_tenant_port
        )
        resp1 = requests.post(mysql_url, json=db_mysql, cookies=auth_cookie, verify=False)
        json_resp1 = resp1.json()
        if json_resp1['code'] == 0:
            log.info("配置MySQL数据库完成")
        else:
            log.error("配置MySQL数据库失败")
            exit(2)

    @status_me("commander")
    def init_tenant(self):
        counter = 0
        succeed = False
        # 重试 10 次，如果还是不成功就报错
        while not succeed and counter < 100:
            time.sleep(5)
            try:
                auth_cookie = self.login_tenant()
                self.tenant_init_mysql(auth_cookie=auth_cookie)
                succeed = True

            except Exception as e:
                # log.error(e)
                succeed = False
                counter += 1
        if not succeed:
            log.error("登录租户系统失败，请检查容器是否启动。")
            exit(2)

    def run(self):
        self.init_commander_mysql()
        self.init_rabbitmq()
        self.init_minio_data()
        self.init_redis()
        self.create_namespace()
        self.push_commander_images()
        self.deploy_istio()
        self.start_commander_service()
        self.commander_proxy_on_nginx()
        self.init_tenant()



from __future__ import absolute_import
from __future__ import unicode_literals

from laipvt.interface.serviceinterface import ServiceInterface
from laipvt.sysutil.util import path_join, status_me


class CaptchaController(ServiceInterface):
    def __init__(self, check_result, service_path):
        super(CaptchaController, self).__init__(check_result, service_path)
        self.project = "mage"

        self.captcha_model_src = path_join(self.data_dir, "verification-tf-serving/model")
        self.captcha_model_remote = path_join(self.deploy_dir, "verification-tf-serving/model")

    @status_me("captcha")
    def prepare_captcha(self):
        self._send_file(src=self.captcha_model_src, dest=self.captcha_model_remote)

    @status_me("captcha")
    def push_captcha_images(self):
        self.push_images(self.project)

    @status_me("captcha")
    def start_captcha_service(self):
        self.start_service(project=self.project, version=self.private_deploy_version)

    def run(self):
        self.prepare_captcha()
        self.push_captcha_images()
        self.start_captcha_service()



from __future__ import absolute_import
from __future__ import unicode_literals

from laipvt.interface.serviceinterface import ServiceInterface
from laipvt.handler.middlewarehandler import EtcdConfigHandler
from laipvt.sysutil.util import path_join, log, status_me
from laipvt.controller.middleware.etcd import EtcdController
from laipvt.sysutil.command import GET_POD_INFO, RESTART_DEPLOYMENT, RESTART_DEPLOYMENT_ALL

class LicenseController(ServiceInterface):
    def __init__(self, check_result, service_path):
        super(LicenseController, self).__init__(check_result, service_path)
        self.namespaces = ["mid", ]
        self.istio_injection_namespaces = ["mid", ]
        self.project = "mid"

    def _prepare_license_data_file(self, is_renew=False, **kwargs):
        if is_renew:
            lcs_file_path = kwargs.get("license_file")
        else:
            lcs_file_path = path_join(self.service_path.data, "license.lcs")

        log.info("找到lcs授权文件: {}".format(lcs_file_path))
        for server in self.nodes:
            data_path = path_join(self.deploy_dir, "license-manager/data")
            self._exec_command_to_host(cmd="mkdir -p {}".format(data_path), server=server)
            self._send_file(src=lcs_file_path, dest=path_join(data_path, "license.lcs"))

    def restart_service(self):
        etcd = EtcdController(self.check_result, EtcdConfigHandler(), "")
        etcd.reset()
        get_pod_info_cmd = GET_POD_INFO.format("license-manager")
        res = self._exec_command_to_host(cmd=get_pod_info_cmd, server=self.harbor_hosts[0], check_res=True)
        if not (res["code"] == 0 and "Running" in res["stdout"]):
            log.error("license服务未获取到，请检查:{}".format(get_pod_info_cmd))
            exit(2)
        restart_deployment_cmd =  RESTART_DEPLOYMENT.format("mid", "license-manager")
        self._exec_command_to_host(cmd=restart_deployment_cmd, server=self.harbor_hosts[0], check_res=True)

        restart_all_deployment_cmd = [
            RESTART_DEPLOYMENT_ALL.format("mage"),
            RESTART_DEPLOYMENT_ALL.format("rpa")
        ]
        self._exec_command_to_host(cmd=restart_all_deployment_cmd, server=self.harbor_hosts[0], check_res=True)

    @status_me("middleware")
    def deploy_license(self):
        self._create_namespace(
            namespaces=self.namespaces,
            istio_injection_namespaces=self.istio_injection_namespaces
        )
        self.push_images(project=self.project)
        self._prepare_license_data_file()
        self.start_service(project=self.project, version=self.private_deploy_version)

    def renew_license(self, license_file):
        self._prepare_license_data_file(is_renew=True, license_file=license_file)
        self.restart_service()

    def deploy(self):
        self.deploy_license()



from __future__ import absolute_import
from __future__ import unicode_literals

import os
import json
from minio import Minio
from laipvt.helper.exception import UtilsError
from laipvt.interface.serviceinterface import ServiceInterface
from laipvt.sysutil.template import FileTemplate
from laipvt.sysutil.util import path_join, log, status_me
from laipvt.sysutil.command import CREATE_DB
from laipvt.sysutil.conf import AccountIdConfig
from laipvt.model.sql import SqlModule


class MageController(ServiceInterface):
    def __init__(self, check_result, service_path):
        super(MageController, self).__init__(check_result, service_path)
        self.namespaces = ["mage", "proxy"]
        self.istio_injection_namespaces = ["mage", "mid", ]
        self.project = "mage"

        self.templates_src = path_join(self.templates_dir, "env_pvt_templates")
        self.common_dest = path_join("/tmp", "env_pvt_common")
        self.common_remote = path_join(self.deploy_dir, "env_pvt_common")

        self.fill_bin_src = path_join(self.templates_dir, "pvt_gen-linux-amd64")
        self.fill_bin_remote = path_join(self.deploy_dir, "pvt_gen-linux-amd64")

        self.mage_config_templates = path_join(self.templates_dir, "mage_conf_templates/Mage")
        self.mage_config_remote = path_join(self.deploy_dir, "mage_conf_templates/Mage")
        self.mage_config_target = path_join(self.deploy_dir, "mage_configmap")
        self.mage_configmap = path_join(self.mage_config_target, "PvtConfig/Mage")
        self.mage_configmap_remote = path_join(self.deploy_dir, "PvtConfig/Mage")

        self.nginx_template = path_join(self.templates_dir, "nginx/http/nginx-mage.tmpl")
        self.nginx_tmp = path_join("/tmp", "nginx-mage.conf")
        self.nginx_file_remote = path_join(self.deploy_dir, "nginx/http/nginx-mage.conf")
        self.minio_data_list = [
            path_join(self.data_dir, "mage_minio_data"),
            path_join(self.data_dir, "siber_minio_data")
        ]

    def _fill_item_file(self):
        log.info("渲染填充项文件{}到{}".format(self.templates_src, self.common_dest))
        try:
            FileTemplate(self.middleware_cfg, self.templates_src, self.common_dest).fill()
        except UtilsError as e:
            log.error(e.msg)
            exit(e.code)
        return True if os.path.isdir(self.common_dest) else False

    def _generic_configmap(self):
        if self._fill_item_file():
            self._send_file(src=self.fill_bin_src, dest=self.fill_bin_remote)
            self._send_file(src=self.common_dest, dest=self.common_remote)
            self._send_file(src=self.mage_config_templates, dest=self.mage_config_remote)

            cmd = [
                "chmod +x {}".format(self.fill_bin_remote),
                "{} -tmplPath={} -valuePath={} -targetPath={}".format(
                    self.fill_bin_remote,
                    self.mage_config_remote,
                    self.common_remote,
                    self.mage_config_target
                )
            ]
            log.info(cmd)
            self._exec_command_to_host(cmd=cmd, server=self.servers[0])

    @status_me("mage")
    def deploy_configmap(self):
        self._generic_configmap()
        self._create_namespace(namespaces=self.namespaces,
                               istio_injection_namespaces=self.istio_injection_namespaces)
        self._send_file(src=self.mage_configmap, dest=self.mage_configmap_remote)
        cmd = "kubectl apply -f {}".format(self.mage_configmap_remote)
        self._exec_command_to_host(cmd=cmd, server=self.harbor_hosts[0])
        self.deploy_istio()

    @status_me("mage")
    def init_minio_data(self):
        try:
            for bucket in self.service_info.buckets:
                # print(bucket)
                try:
                    endpoint = "{}:{}".format(self.middleware_cfg.minio.lb, self.middleware_cfg.minio.nginx_proxy_port)
                    cli = Minio(
                        endpoint,
                        self.middleware_cfg.minio.username,
                        self.middleware_cfg.minio.password,
                        secure=False
                    )
                    if not cli.bucket_exists(bucket):
                        cli.make_bucket(bucket)
                    content_types = {
                        'txt': 'text/plain',
                        'jpg': 'image/jpg',
                        'gif': 'image/gif',
                        'png': 'image/png',
                        'jpeg': 'image/jpeg',
                        'pdf': 'application/pdf',
                        'tif': 'image/tiff',
                        'bmp': 'image/bmp',
                        'xlsx': 'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet'
                    }
                    for data_dir in self.minio_data_list:
                        for i in os.listdir(data_dir):
                            image_name = path_join(data_dir, i)
                            file_type = content_types[i.split('.')[-1]]
                            # cli.fput_object(bucket, i, image_name, content_type=file_type)
                            # if i.split('.')[-1] == "xlsx":
                            cli.fput_object(bucket, "document-mining-backend/" + i,  image_name, content_type=file_type)

                    policy_read_write = {
                        "Version": "2012-10-17",
                        "Statement": [
                            {
                                "Action": [
                                    "s3:GetObject"
                                ],
                                "Effect": "Allow",
                                "Principal": "*",
                                "Resource": [
                                    "arn:aws:s3:::{}/*".format(bucket)
                                ],
                                "Sid": ""
                            }
                        ]
                    }
                    cli.set_bucket_policy(bucket, json.dumps(policy_read_write))

                except Exception as e:
                    log.error(e)
                    log.error("Minio上传数据失败")
                    exit(2)
        except Exception as e:
            log.error(e)
            log.error("创建bucket失败")
            exit(2)

    def create_mage_db_if_exist(self):
        if self.middleware_cfg.mysql.is_deploy:
            mysql_host = self.master_host.ipaddress
        else:
            mysql_host = self.middleware_cfg.mysql.ipaddress[0]
        sql = SqlModule(host=mysql_host, port=int(self.middleware_cfg.mysql.port),
                        user=self.middleware_cfg.mysql.username, passwd=self.middleware_cfg.mysql.password)
        if not sql.select("show databases like '%_saas_docuds';"):
            create_db = CREATE_DB.format(db_name="laiye_saas_docuds")
            sql.insert_sql(create_db)

    @status_me("mage")
    def init_mage_mysql(self):
        log.info("初始化mage mysql")
        version = self.private_deploy_version.split("-")[0]

        self.create_mage_db_if_exist()
        cmd = "docker run --rm --entrypoint /home/works/program/mageTool \
        {}/{}/document-mining-innerservice:{} --operateMode dbnew \
        --host {} --user {} --password {} --port {} --toVersion {}".format(
            self.registry_hub, self.project, self.private_deploy_version,
            self.middleware_cfg.mysql.ipaddress[0], self.middleware_cfg.mysql.username, self.middleware_cfg.mysql.password, self.middleware_cfg.mysql.port,
            version
        )
        res = self._exec_command_to_host(cmd=cmd, server=self.servers[0], check_res=True)
        if res["code"] != 0:
            log.error("初始化mage mysql失败")
            log.error(res["stdout"])
            exit(2)
        log.info("初始化mage mysql完成")
        #self.init_mysql(sql_path=self.service_path.sqls)
        # 渲染siber sqls目录
        # FileTemplate(self.middleware_cfg, self.service_path.siber_sqls, self.service_path.siber_sqls_ok).fill()
        # self.init_mysql(sql_path=self.service_path.siber_sqls_ok)

    @status_me("mage")
    def mage_transfer_data(self):
        src_dir_path = path_join(self.service_path.data, "mage_transfer_data")
        dest_dir_path = path_join(self.deploy_dir, "mage_transfer_data")
        image = "{}/{}/document-mining-innerservice:{}".format(self.registry_hub, self.project, self.private_deploy_version)
        version = self.private_deploy_version.split("-")[0]
        # cmd = "docker run -v {dir_path}:/transfer_data --entrypoint /home/works/program/transferApp {image} \
        #  --dataMode 2 --host {mysql_host} --port {mysql_port} --user {mysql_user} --password {mysql_password} \
        #  --minioAddr http://{minio_host}:{minio_port} -dir /transfer_data".format(
        #     dir_path=dest_dir_path, image=image,
        #     mysql_host=self.middleware_cfg.mysql.ipaddress[0],
        #     minio_port=self.middleware_cfg.minio.port,
        #     mysql_port=self.middleware_cfg.mysql.port,
        #     mysql_user=self.middleware_cfg.mysql.username,
        #     mysql_password=self.middleware_cfg.mysql.password,
        #     minio_host=self.middleware_cfg.minio.lb
        # )
        cmd = "docker run --rm --entrypoint /home/works/program/mageTool {image} \
         --operateMode datatransfer --dataMode 2 --minioAddr http://{minio_host}:{minio_port} \
          --host {mysql_host} --user {mysql_user} --password {mysql_password} --port {mysql_port} \
          --toVersion {version}".format(
            image=image, minio_host=self.middleware_cfg.minio.lb, minio_port=self.middleware_cfg.minio.port,
            mysql_host=self.middleware_cfg.mysql.ipaddress[0],mysql_user=self.middleware_cfg.mysql.username,
            mysql_password=self.middleware_cfg.mysql.password, mysql_port=self.middleware_cfg.mysql.port,
            version=version
        )
        self._send_file(src=src_dir_path, dest=dest_dir_path)
        res = self._exec_command_to_host(cmd=cmd, server=self.servers[0], check_res=True)
        log.info("userid: {}".format(res["stdout"].split("\n")[-2]))
        AccountIdConfig().save(res["stdout"].split("\n")[-2])

    @status_me("mage")
    def push_mage_images(self):
        self.push_images(self.project)

    @status_me("mage")
    def start_mage_service(self):
        self.start_service(project=self.project, version=self.private_deploy_version)

    @status_me("mage")
    def mage_proxy_on_nginx(self):
        self.proxy_on_nginx(self.nginx_template, self.nginx_tmp, self.nginx_file_remote)

    def run(self):
        self.push_mage_images()
        self.init_mage_mysql()
        self.init_identity_user()
        self.init_minio_data()
        self.deploy_configmap()
        self.mage_transfer_data()
        self.start_mage_service()
        self.mage_proxy_on_nginx()



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
import os
import json
from laipvt.sysutil.gvalue import CHECK_FILE, PROJECT_INFO_FILE
from laipvt.sysutil.util import find
from laipvt.handler.confighandler import CheckResultHandler
from laipvt.handler.packagehandler import DeployPackageHandler
from laipvt.controller.kubernetes.kube import KubeController
from laipvt.controller.middleware.harbor import HarborController
from laipvt.controller.middleware.nginx import NginxController
from laipvt.controller.middleware.etcd import EtcdController
from laipvt.controller.middleware.minio import MinioController
from laipvt.controller.middleware.redis import RedisController
from laipvt.controller.middleware.mysql import MysqlController
from laipvt.controller.middleware.elasticsearch import EsController
from laipvt.controller.middleware.rabbitmq import RabbitmqController
from laipvt.controller.middleware.identity import IdentityController
from laipvt.controller.middleware.siber import SiberController
from laipvt.controller.service.license import LicenseController
from laipvt.controller.service.mage import MageController
from laipvt.controller.service.ocr_standard import OcrStandardController
from laipvt.controller.service.nlp import NlpController
from laipvt.controller.service.seal import SealController
from laipvt.controller.service.captcha import CaptchaController
from laipvt.controller.service.commander import CommanderController
from laipvt.controller.service.ocr import OcrController
from laipvt.controller.middleware.monitor import MonitorController
from laipvt.controller.middleware.keepalived import KeepalivedController
from laipvt.handler.middlewarehandler import EtcdConfigHandler, MysqlConfigHandler, EsConfigHandler, \
    MinioConfigHandler, RabbitmqConfigHandler, RedisConfigHandler, HarborConfigHandler, NginxConfigHandler, \
    IdentityConfigHandler, SiberConfigHandler, OcrConfigHandler, MonitorConfigHandler, KeepalivedConfigHandler
from laipvt.sysutil.relation import module_require_tfserver, tfserver_name, tfserver_image_name
from laipvt.sysutil.util import write_to_file, read_form_json_file, gen_https_self_signed_ca
from laipvt.sysutil.kube_common import wait_pod_running
from laipvt.sysutil.check import is_pre_check, check_use_https_ca, check_https_ca_self_signed

def deploy_main(args):
    if not args.isCheck:
        if not is_pre_check():
            print("ERROR: 未执行前置检查或前置检查失败")
            exit(1)

    if not check_use_https_ca():
        print("Error: 使用https部署，选择的是客户提供证书，证书路径不正确")
        exit(1)

    if check_https_ca_self_signed():
        gen_https_self_signed_ca()

    # 获取前置检查结果
    check_result_file = CHECK_FILE
    check_result = CheckResultHandler(check_result_file)

    if args.targzFile:
        pkg_path = False
        if not os.path.exists(args.targzFile):
            cwd = [os.getcwd(), check_result.deploy_dir]
            for d in cwd:
                pkg_path = find(d, args.targzFile, file=True)
                if pkg_path:
                    break
        else:
            pkg_path = os.path.join(os.getcwd(), args.targzFile)
        if not pkg_path:
            print("未找到文件")
            exit(1)
        PKG = os.path.dirname(pkg_path)
        ID = os.path.basename(pkg_path).split(".")[0]
        PKG_DIR = pkg_path.split(".")[0]

        # 将项目ID和path写入文件缓存
        project_dict = { "PKG": PKG, "ID": ID }
        write_to_file(PROJECT_INFO_FILE, json.dumps(project_dict, indent=4))

        deploy_package = DeployPackageHandler(PKG, ID)
        if not os.path.exists(PKG_DIR):
            deploy_package.unpack()
        # 解析
        parse_package = deploy_package.parse()
        kubernetes_package = parse_package.kubernetes
        middleware_package = parse_package.middleware
        harbor_package = parse_package.harbor

        if not os.path.exists(os.path.join(PKG_DIR, "kubernetes")):
            kubernetes_package.kubernetes_unpack()
        if not os.path.exists(os.path.join(PKG_DIR, "middleware")):
            middleware_package.unpack()
        if not os.path.exists(os.path.join(PKG_DIR, "harbor")):
            harbor_package.unpack()

        # install harbor
        haror_path = harbor_package.parse().harbor
        harbor_config = HarborConfigHandler()
        harbor = HarborController(check_result, harbor_config, haror_path)
        harbor.install_harbor()

        # install nginx
        nginx_package = middleware_package.parse().nginx
        nginx_config = NginxConfigHandler()
        nginx = NginxController(check_result, nginx_config, nginx_package)
        nginx.install_nginx()

        # install kubernetes
        kube_info = kubernetes_package.parse()
        kube = KubeController(check_result, kube_info)
        kube.add_hosts()
        kube.system_prepare()
        kube.init_primary_master()
        kube.cp_kube_config()
        kube.kube_completion()
        kube.install_network_plugin()
        kube.join_master()
        kube.join_node()
        kube.install_helm()
        kube.install_istio()
        # 更新nginx服务tcp代理apiserever cluster
        nginx.renew_apiserver_config()


        # install service
        services = {
            "mage": MageController,
            "commander": CommanderController,
            "nlp": NlpController,
            "captcha": CaptchaController,
            "ocr_standard": OcrStandardController,
            "ocr": OcrController,
            "seal": SealController
        }

        middlewares = {
            "etcd": (EtcdConfigHandler, EtcdController),
            "minio": (MinioConfigHandler, MinioController),
            "redis": (RedisConfigHandler, RedisController),
            "mysql": (MysqlConfigHandler, MysqlController),
            "es": (EsConfigHandler, EsController),
            "rabbitmq": (RabbitmqConfigHandler, RabbitmqController),
            # "identity": (IdentityConfigHandler, IdentityController)
        }

        for s in parse_package.service:
            if not os.path.exists(os.path.join(PKG_DIR, s.project_name)):
                s.unpack()
            service_path = s.parse()

            # 遍历服务需要的中间件，依次安装
            middleware_list = service_path.config.middleware
            middleware_list.insert(0, "etcd")

            for mid in middleware_list:
                path = middleware_package.parse()[mid]
                config = middlewares[mid][0]()
                middleware = middlewares[mid][1](check_result, config, path)
                middleware.deploy()

            # install identity
            identity_path = middleware_package.parse().identity
            identity_config = IdentityConfigHandler()
            identity = IdentityController(check_result, identity_config, identity_path)
            identity.deploy_identity()

            # install license
            license_package = parse_package.license
            if not os.path.exists(os.path.join(PKG_DIR, "license")):
                license_package.unpack()
            license_path = license_package.parse()
            license = LicenseController(check_result, license_path)
            license.deploy_license()

            if s.project_name == "ocr":
                ocr_handler = OcrConfigHandler()
                deploy_service = services[s.project_name](service_path, check_result, ocr_handler, s.root_dir)
            else:
                deploy_service = services[s.project_name](check_result, service_path)
            if s.project_name == "commander":
                identity_path = middleware_package.parse().identity
                identity_config = IdentityConfigHandler()
                identity = IdentityController(check_result, identity_config, identity_path)
                identity.update_identity_config()
            deploy_service.run()

            # tf-server
            if s.project_name in module_require_tfserver:
                for module_name in tfserver_name[s.project_name]:
                    deploy_service.deploy_tf_service(module_name, tfserver_image_name[s.project_name])

        # check all pod status
        if not wait_pod_running:
            print("kubernetes集群中有pod启动状态异常，请检查: kubectl get pod -A")
            exit(2)

        # install siber
        siber_path = middleware_package.parse().siber
        siber_config = SiberConfigHandler()
        siber = SiberController(check_result, siber_config, siber_path)
        siber.deploy_siber()
        for s in parse_package.service:
            if s.project_name == "mage":
                siber.replace_mage_collection_tag(parse_package.config.siber_tags)
            elif s.project_name == "commander":
                exit()

    if args.which == "license":
        if args.LicenseFile:
            if os.path.exists(args.LicenseFile):
                # print(args.LicenseFile)
                project_dict = read_form_json_file(PROJECT_INFO_FILE)

                # 解析大包
                deploy_package = DeployPackageHandler(project_dict["PKG"], project_dict["ID"])
                parse_package = deploy_package.parse()
                license_package = parse_package.license
                # license_package.unpack()
                license_path = license_package.parse()
                license = LicenseController(check_result, license_path)
                license.renew_license(license_file=args.LicenseFile)
            else:
                print("请检查指定的新授权文件是否存在: {}".format(args.LicenseFile))
                exit(1)

        if args.OcrLicenseFile:
            if os.path.exists(args.OcrLicenseFile):
                project_dict = read_form_json_file(PROJECT_INFO_FILE)

                deploy_package = DeployPackageHandler(project_dict["PKG"], project_dict["ID"])
                parse_package = deploy_package.parse()
                renew_flag = False
                for s in parse_package.service:
                    if s.project_name == "ocr":
                        service_path = s.parse()
                        ocr_handler = OcrConfigHandler()
                        deploy_ocr = OcrController(service_path, check_result, ocr_handler, s.root_dir)
                        deploy_ocr.renew_license(license_file=args.OcrLicenseFile)
                        renew_flag = True

                if not renew_flag:
                    print("OCR license未更新成功，请检查是否存在合合服务")
                    exit(2)

    if args.which == "add":
        if args.Monitor:
            project_dict = read_form_json_file(PROJECT_INFO_FILE)
            deploy_package = DeployPackageHandler(project_dict["PKG"], project_dict["ID"])
            parse_package = deploy_package.parse()
            middleware_package = parse_package.middleware

            monitor_path = middleware_package.parse().monitor
            monitor_config = MonitorConfigHandler()
            monitor = MonitorController(check_result, monitor_config, monitor_path)
            monitor.deploy_monitor()

        if args.Keepalive:
            project_dict = read_form_json_file(PROJECT_INFO_FILE)
            deploy_package = DeployPackageHandler(project_dict["PKG"], project_dict["ID"])
            parse_package = deploy_package.parse()
            middleware_package = parse_package.middleware

            keepalive_path = middleware_package.parse().keepalived
            keepalive_config = KeepalivedConfigHandler()
            keepalive = KeepalivedController(check_result, keepalive_config, keepalive_path)
            keepalive.deploy_keepalived()



#!/bin/env python
# -*- encoding: utf-8 -*-
import json
import os
from laipvt.sysutil.conf import status_file


dict_tmpl = {
    "basesystem": {
        "unpack": 0,
        "kubernetes_unpack": 0,
        "install_harbor": 0,
        "install_nginx": 0,
        "add_hosts": 0,
        "install_rpms": 0,
        "system_prepare": 0,
        "init_primary_master": 0,
        "kube_completion": 0,
        "install_network_plugin": 0,
        "join_master": 0,
        "join_node": 0,
        "install_helm": 0,
        "install_istio": 0
    },
    "middleware": {
        "deploy_etcd": 0,
        "deploy_license": 0,
        "deploy_minio": 0,
        "deploy_mysql": 0,
        "deploy_redis": 0,
        "deploy_es": 0,
        "deploy_rabbitmq": 0,
        "deploy_identity": 0,
        "deploy_commander_identity": 0,
        "deploy_monitor": 0,
        "deploy_keepalived": 0,
        "deploy_siber": 0
    },
    "mage": {
        "init_mage_mysql": 0,
        "init_identity_user": 0,
        "init_minio_data": 0,
        "push_mage_images": 0,
        "deploy_configmap": 0,
        "start_mage_service": 0,
        "mage_proxy_on_nginx": 0,
        "mage_transfer_data": 0
    },
    "nlp": {
        "prepare_nlp": 0,
        "push_nlp_images": 0,
        "start_nlp_service": 0
    },
    "ocr": {
        "gen_middleware_conf": 0,
        "prepare_ocr": 0,
        "install_ocr": 0
    },
    "captcha": {
        "status": 0,
        "gen_middleware_conf": 0,
        "push_captcha_images": 0,
        "prepare_captcha": 0,
        "install_captcha": 0
    },
    "commander": {
        "status": 0,
        "gen_commander_conf": 0,
        "install": 0,
        "install_harbor": 0,
        "install_nginx": 0,
        "init_rabbitmq": 0,
        "init_redis": 0,
        "init_mysql": 0,
        "init_minio": 0,
        "preparation": 0,
        "istio_proxy": 0,
        "proxy_nginx": 0,
        "init_tenant": 0,
        "apitest": 0
    }
}


class Status:
    def __init__(self):
        self.status_file = status_file
        self.STATUS_SUCCESS = 1
        self.STATUS_FAILED = 2
        self.STATUS_NOT_RUNNING = 0
        self.status_dicts = [self.STATUS_NOT_RUNNING, self.STATUS_SUCCESS, self.STATUS_FAILED]
        if os.path.exists(self.status_file):
            with open(self.status_file, 'r') as fp:
                self.status = json.load(fp)
        else:
            with open(self.status_file, 'a') as fp:
                fp.write(json.dumps(dict_tmpl, indent=4))
            self.status = dict_tmpl

    def reset_status(self):
        status = json.loads(json.dumps(dict_tmpl))
        with open(self.status_file, "w") as sf:
            json.dump(status, sf, indent=4)

    def _reload(self):
        with open(self.status_file) as sf:
            self.status = json.load(sf)

    def _update(self):
        with open(self.status_file, "w") as sf:
            json.dump(self.status, sf, indent=4)
        self._reload()

    def get_status_failed(self, project_name):
        step_list = []
        proj = self.status[project_name]
        for step in proj:
            if proj[step] == self.STATUS_FAILED:
                step_list.append(step)
        return step_list

    def get_status(self, project, step):
        try:
            self._reload()
            return self.status_dicts[self.status[project][step]]
        except KeyError:
            self.update_status(project, step, 0)
            self._reload()
            return self.status_dicts[self.status[project][step]]

    def update_status(self, project, step, value):
        try:
            self.status[project][step] = int(self.status_dicts[value])
            self._update()
            return True
        except IndexError:
            return False



from __future__ import absolute_import
from __future__ import unicode_literals

import os
import re
import json
import time
from shutil import copyfile
from ruamel import yaml
from laipvt.sysutil.gvalue import STATUS_FILE, LAIPVT_BASE_DIR
from laipvt.helper.exception import UtilsError

base_dir = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
status_file = STATUS_FILE


class ConfigInterface(object):
    """
        读取、写入文件通用接口
        读取文件调用read_file, 读取目录下文件调用read_dir
    """
    def __init__(self, path: str, data="", suffix=""):
        """
        :param path: 文件路径，可以是文件或者目录
        :param data: 要写入的数据，默认为空
        :param suffix: 如果path是目录，输入要匹配的文件结尾
        """
        self.data = data
        self.file_path = path
        self.multiple_file_path = []
        self.suffix = suffix
        self.file_suffix_regex = '.*\.%s$' % self.suffix

    def data_loader(self) -> str:
        pass

    def loader(self, p) -> dict:
        pass

    def writer(self):
        pass

    def multiple_loader(self):
        data_dict = {}
        for f in os.listdir(self.file_path):
            if re.match(self.file_suffix_regex, f):
                self.multiple_file_path.append(f)
        for real_file in self.multiple_file_path:
            try:
                data_dict.update(self.loader(os.path.join(self.file_path, real_file)))
            except Exception as e:
                raise UtilsError("读取文件{}失败".format(os.path.join(self.file_path, real_file)))
        return data_dict

    def read_from_data(self) -> str:
        return self.data_loader()

    def read_file(self) -> dict:
        try:
            if os.path.isfile(self.file_path):
                return self.loader(self.file_path)
            raise UtilsError("%s 文件不存在" % self.file_path)
        except json.decoder.JSONDecodeError:
            raise UtilsError("ConfigInterface.read_file 不是一个合法的json文件") from None

    def read_dir(self) -> dict:
        try:
            if os.path.isdir(self.file_path):
                return self.multiple_loader()
        except Exception as e:
            raise UtilsError("ConfigInterface.read_dir {}".format(e)) from None

    def backup_file(self, path):
        try:
            if os.path.isdir(path):
                backup_file_suffix = ".bak-%d" % int(time.time() * 1000)
                file_name = os.path.split(self.file_path)[1]
                backup_file = os.path.join(path, file_name + backup_file_suffix)
                copyfile(self.file_path, backup_file)
            else:
                raise UtilsError("%s 不是一个目录" % path)
        except Exception:
            raise UtilsError("ConfigInterface.backup_file") from None

    def write_file(self, backup=True, backup_path="/tmp"):
        if backup:
            self.backup_file(backup_path)
        self.writer()


class YamlConfig(ConfigInterface):

    def __init__(self, path, data="", suffix="yaml"):
        super().__init__(path, data, suffix=suffix)

    def data_loader(self) -> str:
        return yaml.dump(self.data)

    def loader(self, p):
        with open(p, "r", encoding="utf-8") as f:
            return yaml.load(f, Loader=yaml.SafeLoader)

    def writer(self):
        if not os.path.exists(os.path.dirname(self.file_path)):
            os.makedirs(os.path.dirname(self.file_path))
        with open(self.file_path, "w+", encoding="utf-8") as f:
            yaml.dump(self.data, f,
                      Dumper=yaml.RoundTripDumper, default_flow_style=False, allow_unicode=True, indent=4)

class JsonConfig(ConfigInterface):

    def __init__(self, path, data="", suffix="json"):
        super().__init__(path, data, suffix=suffix)

    def data_loader(self) -> str:
        return json.dumps(self.data)

    def loader(self, p):
        with open(p, "r", encoding="utf-8") as f:
            return json.load(f)

    def writer(self):
        if not os.path.exists(os.path.dirname(self.file_path)):
            os.makedirs(os.path.dirname(self.file_path))
        with open(self.file_path, "w+", encoding="utf-8") as f:
            json.dump(self.data, f,
                      ensure_ascii=False, separators=(',', ':'), indent=4)

class AccountIdConfig(object):

    @classmethod
    def save(cls, account_id):
        with open(os.path.join(LAIPVT_BASE_DIR, "accountid.txt"), "w+", encoding="utf-8") as f:
            f.write(account_id)

    @classmethod
    def get(cls):
        with open(os.path.join(LAIPVT_BASE_DIR, "accountid.txt"), "r", encoding="utf-8") as f:
            content = f.read()
            return content.strip()



# -*- coding:utf8 -*-
# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
import grpc

from laipvt.sysutil import dboperation_pb2 as protos_dot_docUnderstanding_dot_backend_dot_dboperation__pb2


class DbOperationServiceStub(object):
  """db操作服务
  """

  def __init__(self, channel):
    """Constructor.
    Args:
      channel: A grpc.Channel.
    """
    self.Operate = channel.unary_unary(
        '/docUnderstanding.DbOperationService/Operate',
        request_serializer=protos_dot_docUnderstanding_dot_backend_dot_dboperation__pb2.DbOperationRequest.SerializeToString,
        response_deserializer=protos_dot_docUnderstanding_dot_backend_dot_dboperation__pb2.DbOperationResponse.FromString,
        )


class DbOperationServiceServicer(object):
  """db操作服务
  """

  def Operate(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')


def add_DbOperationServiceServicer_to_server(servicer, server):
  rpc_method_handlers = {
      'Operate': grpc.unary_unary_rpc_method_handler(
          servicer.Operate,
          request_deserializer=protos_dot_docUnderstanding_dot_backend_dot_dboperation__pb2.DbOperationRequest.FromString,
          response_serializer=protos_dot_docUnderstanding_dot_backend_dot_dboperation__pb2.DbOperationResponse.SerializeToString,
      ),
  }
  generic_handler = grpc.method_handlers_generic_handler(
      'docUnderstanding.DbOperationService', rpc_method_handlers)
  server.add_generic_rpc_handlers((generic_handler,))


# -*- coding:utf8 -*-
# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: protos/docUnderstanding/backend/dboperation.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='protos/docUnderstanding/backend/dboperation.proto',
  package='docUnderstanding',
  syntax='proto3',
  serialized_options=_b('ZHgit.laiye.com/laiye-backend-repos/im-saas-protos-golang/docUnderstanding'),
  serialized_pb=_b('\n1protos/docUnderstanding/backend/dboperation.proto\x12\x10\x64ocUnderstanding\"!\n\x12\x44\x62OperationRequest\x12\x0b\n\x03sql\x18\x01 \x01(\t\"F\n\x0b\x44\x62\x44\x61taField\x12\x0b\n\x03key\x18\x01 \x01(\t\x12*\n\x04type\x18\x02 \x01(\x0e\x32\x1c.docUnderstanding.DbDataType\"#\n\x11\x44\x62OperationRecord\x12\x0e\n\x06values\x18\x01 \x03(\t\"z\n\x13\x44\x62OperationResponse\x12-\n\x06\x66ields\x18\x01 \x03(\x0b\x32\x1d.docUnderstanding.DbDataField\x12\x34\n\x07records\x18\x02 \x03(\x0b\x32#.docUnderstanding.DbOperationRecord*%\n\nDbDataType\x12\x17\n\x13\x44\x42_DATA_TYPE_STRING\x10\x00\x32n\n\x12\x44\x62OperationService\x12X\n\x07Operate\x12$.docUnderstanding.DbOperationRequest\x1a%.docUnderstanding.DbOperationResponse\"\x00\x42JZHgit.laiye.com/laiye-backend-repos/im-saas-protos-golang/docUnderstandingb\x06proto3')
)

_DBDATATYPE = _descriptor.EnumDescriptor(
  name='DbDataType',
  full_name='docUnderstanding.DbDataType',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='DB_DATA_TYPE_STRING', index=0, number=0,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=339,
  serialized_end=376,
)
_sym_db.RegisterEnumDescriptor(_DBDATATYPE)

DbDataType = enum_type_wrapper.EnumTypeWrapper(_DBDATATYPE)
DB_DATA_TYPE_STRING = 0



_DBOPERATIONREQUEST = _descriptor.Descriptor(
  name='DbOperationRequest',
  full_name='docUnderstanding.DbOperationRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='sql', full_name='docUnderstanding.DbOperationRequest.sql', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=71,
  serialized_end=104,
)


_DBDATAFIELD = _descriptor.Descriptor(
  name='DbDataField',
  full_name='docUnderstanding.DbDataField',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='docUnderstanding.DbDataField.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='type', full_name='docUnderstanding.DbDataField.type', index=1,
      number=2, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=106,
  serialized_end=176,
)


_DBOPERATIONRECORD = _descriptor.Descriptor(
  name='DbOperationRecord',
  full_name='docUnderstanding.DbOperationRecord',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='values', full_name='docUnderstanding.DbOperationRecord.values', index=0,
      number=1, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=178,
  serialized_end=213,
)


_DBOPERATIONRESPONSE = _descriptor.Descriptor(
  name='DbOperationResponse',
  full_name='docUnderstanding.DbOperationResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='fields', full_name='docUnderstanding.DbOperationResponse.fields', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='records', full_name='docUnderstanding.DbOperationResponse.records', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=215,
  serialized_end=337,
)

_DBDATAFIELD.fields_by_name['type'].enum_type = _DBDATATYPE
_DBOPERATIONRESPONSE.fields_by_name['fields'].message_type = _DBDATAFIELD
_DBOPERATIONRESPONSE.fields_by_name['records'].message_type = _DBOPERATIONRECORD
DESCRIPTOR.message_types_by_name['DbOperationRequest'] = _DBOPERATIONREQUEST
DESCRIPTOR.message_types_by_name['DbDataField'] = _DBDATAFIELD
DESCRIPTOR.message_types_by_name['DbOperationRecord'] = _DBOPERATIONRECORD
DESCRIPTOR.message_types_by_name['DbOperationResponse'] = _DBOPERATIONRESPONSE
DESCRIPTOR.enum_types_by_name['DbDataType'] = _DBDATATYPE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

DbOperationRequest = _reflection.GeneratedProtocolMessageType('DbOperationRequest', (_message.Message,), dict(
  DESCRIPTOR = _DBOPERATIONREQUEST,
  __module__ = 'protos.docUnderstanding.backend.dboperation_pb2'
  # @@protoc_insertion_point(class_scope:docUnderstanding.DbOperationRequest)
  ))
_sym_db.RegisterMessage(DbOperationRequest)

DbDataField = _reflection.GeneratedProtocolMessageType('DbDataField', (_message.Message,), dict(
  DESCRIPTOR = _DBDATAFIELD,
  __module__ = 'protos.docUnderstanding.backend.dboperation_pb2'
  # @@protoc_insertion_point(class_scope:docUnderstanding.DbDataField)
  ))
_sym_db.RegisterMessage(DbDataField)

DbOperationRecord = _reflection.GeneratedProtocolMessageType('DbOperationRecord', (_message.Message,), dict(
  DESCRIPTOR = _DBOPERATIONRECORD,
  __module__ = 'protos.docUnderstanding.backend.dboperation_pb2'
  # @@protoc_insertion_point(class_scope:docUnderstanding.DbOperationRecord)
  ))
_sym_db.RegisterMessage(DbOperationRecord)

DbOperationResponse = _reflection.GeneratedProtocolMessageType('DbOperationResponse', (_message.Message,), dict(
  DESCRIPTOR = _DBOPERATIONRESPONSE,
  __module__ = 'protos.docUnderstanding.backend.dboperation_pb2'
  # @@protoc_insertion_point(class_scope:docUnderstanding.DbOperationResponse)
  ))
_sym_db.RegisterMessage(DbOperationResponse)


DESCRIPTOR._options = None

_DBOPERATIONSERVICE = _descriptor.ServiceDescriptor(
  name='DbOperationService',
  full_name='docUnderstanding.DbOperationService',
  file=DESCRIPTOR,
  index=0,
  serialized_options=None,
  serialized_start=378,
  serialized_end=488,
  methods=[
  _descriptor.MethodDescriptor(
    name='Operate',
    full_name='docUnderstanding.DbOperationService.Operate',
    index=0,
    containing_service=None,
    input_type=_DBOPERATIONREQUEST,
    output_type=_DBOPERATIONRESPONSE,
    serialized_options=None,
  ),
])
_sym_db.RegisterServiceDescriptor(_DBOPERATIONSERVICE)

DESCRIPTOR.services_by_name['DbOperationService'] = _DBOPERATIONSERVICE

# @@protoc_insertion_point(module_scope)


from __future__ import absolute_import
from __future__ import unicode_literals

from functools import reduce
from laipvt.sysutil.util import get_yaml_obj, get_json_obj, get_yaml_config, to_object
from laipvt.handler import ListHandler, DictHandler
from laipvt.sysutil.relation import service_module_relation, machine_module_relation, get_module_key

class ProjectsHandler(ListHandler):
    def __init__(self, projects):
        super(ProjectsHandler, self).__init__(projects)
        self.mage = self.check("mage")
        self.commander = self.check("commander")
        self.wulai = self.check("wulai")

class RoleHandler(ListHandler):
    def __init__(self, role):
        super(RoleHandler, self).__init__(role)
        self.master = self.check("master")
        self.node = self.check("node")
        self.harbor = self.check("harbor")
        self.licserver = self.check("licserver")

class ServerHandler(DictHandler):
    """返回服务器信息"""
    def __init__(self, server):
        super(ServerHandler, self).__init__(server)
        self.ipaddress = self.d.ipaddress
        self.username = self.d.username
        self.password = self.d.password
        self.port = self.d.port
        self.role = RoleHandler(self.d.role)

class ServersHandler(ListHandler):
    """返回服务器列表"""
    def __init__(self, servers):
        super(ServersHandler, self).__init__(servers)
        try:
            self.servers = self.get()
        except KeyError:
            self.servers = to_object(self.l)

    def get(self) -> list:
        """

        :return: list[ServerHandler]
        """
        l = []
        for server in self.l:
            l.append(ServerHandler(server))
        return l

    def get_role_ip(self, role: str) -> list:
        """根据角色获取匹配该角色的ip地址列表"""
        l = []
        for server in self.servers:
            if server.role.check(role):
                l.append(server.ipaddress)
        return l

    def get_all_ip(self) -> list:
        """根据角色获取匹配该角色的ip地址列表"""
        l = []
        for server in self.servers:
            l.append(server.ipaddress)
        return l

    def get_role_obj(self, role: str) -> list:
        l = []
        for server in self.servers:
            if server.role.check(role):
                l.append(server)
        return l

    def search_server(self, key: str, value: str) -> list:
        l = []
        for server in self.servers:
            if server.__dict__[key] == value:
                l.append(server)
        return l

class CheckResultHandler():
    def __init__(self, config_file: str):
        self.config = get_yaml_obj(config_file)
        self.deploy_dir = self.config.deploy_dir
        self.lb = self.config.lb
        self.deploy_projects = ProjectsHandler(self.config.deploy_projects)
        self.servers = ServersHandler(self.config.servers)
        self.licserver = self.config.licserver

class DeployServiceHandler():
    """
    处理要部署的项目，返回字典
    service: 自研的项目
    ocr: ocr相关
    """
    def __init__(self, deploy_service: list):
        res = []
        for id in deploy_service:
            if id in service_module_relation["ocr_standard"]:
                res.append("ocr_standard")
            elif id in service_module_relation["ocr"]:
                res.append("ocr")
            else:
                res.append(get_module_key(id))
            # for srv in service_module_relation:
            #     if int(id) in service_module_relation[srv]:
            #         res.append(srv)
        self.service_id = deploy_service
        self.service_list = reduce(lambda x, y: x if y in x else x + [y], [[], ] + res)

    def get(self) -> list:
        return self.service_list

    def parse(self, l: list) -> dict:
        cpu_list = []
        gpu_list = []
        for id in l:
            if id in machine_module_relation['cpu']:
                cpu_list.append(id)
            elif id in machine_module_relation['gpu']:
                gpu_list.append(id)
        return {
            "cpu": cpu_list,
            "gpu": gpu_list
        }

class PvtAdminConfigHandler():
    """解析私有部署授权平台传递配置
    {
        "code_version": 01,
        "deploy_type": simple/ha
        "project_name": "项目名称",
        "project_id": "abcde",
        "start_time": 1606199550,
        "end_time": 1608791570,
        "license_file_name": "abcde.lcs",
        "deploy_service": [0, 1, 2, 3, 4, 5]
    }
    {
            "commander": ["commander"],
            "mage": ["mage", "nlp", "captcha", "laiye_ocr", "hehe_ocr"],
            "wulai": ["wulai"],
            "laiye_ocr"li: ["laiye_document_cpu", "laiye_document_gpu", "laiye_table_cpu", "laiye_table_gpu"],
            "hehe_ocr": ["hehe_document_cpu", "hehe_document_gpu", "hehe_table_cpu", "hehe_table_gpu"],
    }
    """
    def __init__(self, config: str):
        self.config = get_json_obj(config)
        self.code_version = self.config.code_version
        self.project_name = self.config.project_name
        self.project_id = self.config.project_id
        self.license_file_name = self.config.license_file_name
        self.service_list = DeployServiceHandler(self.config.deploy_service)
        try:
            self.siber_tags = self.config.siber_tags
        except Exception:
            pass


class ServiceConfigHandler:
    """
    middleware:
      - mysql
      - redis
      - minio
    services:
      document-mining-backend:
        - document-mining-rpc
        - document-mining-auth
        - document-mining-openapi
      file-analyze:
        - file-analyze
      ocr-server-dispatch:
        - ocr-server-dispatch
    """
    def __init__(self, conf: str):
        self.conf = get_yaml_config(conf)
        self.middleware = self.conf.get("middleware", [])
        self.services = self.conf.get("services", [])
        self.tag = self.conf.get("tag", "")
        self.cfg = to_object({})
        self.buckets = self.conf.get("buckets", [])
        self.require_tfserver = self.conf.get("require_tfserver", False)
        self.machine_type = self.conf.get("machine_type", "cpu")

    def get_process(self, service: str) -> list:
        return self.services.get(service)



from __future__ import absolute_import
from __future__ import unicode_literals

import os
from laipvt.sysutil.gvalue import DEPLOY_LANGUAGE

class Error():
    def __init__(self):
        lang = os.environ.get("DEPLOY_LANGUAGE", DEPLOY_LANGUAGE)
        map = {
            "en": self.en,
            "cn": self.cn
        }
        try:
            map[lang]()
        except KeyError:
            self.cn()

    def cn(self):
        self.UNIMPLEMENT_ERROR = "未实现的方法: {}"
        self.USER_EXIT = "用户手动退出。程序结束"

    def en(self):
        self.UNIMPLEMENT_ERROR = "un implement function: {}"
        self.USER_EXIT = "user exit. quit process"

class PackageErrors(Error):
    def __init__(self):
        super(PackageErrors, self).__init__()

    def cn(self):
        self.PROJECT_ID_UNMATCH = "授权id与实际申请的项目不匹配"
        self.PROJECT_ID_INCORRECT = "授权id不正确"
        self.PACKAGE_ILLEGAL = "文件不合法，{}不存在"
        self.UNPACK_ERROR = "文件解压失败，错误信息: {}"
        self.PACK_ERROR = "文件打包失败，错误信息: {}"

    def en(self):
        self.PROJECT_ID_UNMATCH = "authorization id is unmatch with this project"
        self.PROJECT_ID_INCORRECT = "incorrect authorization id"
        self.PACKAGE_ILLEGAL = "file type illegal，{} is not exist"
        self.UNPACK_ERROR = "file unpack failed, error msg: {}"
        self.PACK_ERROR = "package file failed, error msg: {}"

class FileTypeErrors(Error):
    def __init__(self):
        super(FileTypeErrors, self).__init__()

    def cn(self):
        self.WRONG_FILE_TYPE = "不支持的文件类型"

    def en(self):
        self.WRONG_FILE_TYPE = "unsupported file type"

class RuntimeErrors(Error):
    def __init__(self):
        super(RuntimeErrors, self).__init__()

    def cn(self):
        self.SEND_FILE_ERROR = "发送文件{}, 到服务器: {} 失败。错误信息: {}"
        self.RUN_CMD_ERROR = "在服务器: {} 执行命令 {} 失败。错误信息: {}"

    def en(self):
        self.SEND_FILE_ERROR = "Failed send {} to server: {}. Error message: {}"
        self.RUN_CMD_ERROR = "Server: {}. Run command {} failed. Error message: {}"


from __future__ import absolute_import
from __future__ import unicode_literals

import os
import sys
import paramiko
import tarfile
import time
import re
import socket
import subprocess
import shutil
from laipvt.helper.exception import SSHAuthFailed, SSHAuthInfoMissed, SSHConnectFailed, SSHPortFailed, \
    SSHPortOrIpIllegal
from laipvt.helper.errors import FileTypeErrors
from paramiko.ssh_exception import AuthenticationException, NoValidConnectionsError


def monkey_sudo(password, cmd):
    cmd = "echo '{}' | sudo -S {}".format(password, cmd)
    return cmd


def to_str(bytes_or_str):
    """
    把byte类型转换为str
    :param bytes_or_str:
    :return:
    """
    if isinstance(bytes_or_str, bytes):
        value = bytes_or_str.decode('utf-8')
    else:
        value = bytes_or_str
    return value


class SSHConnect(object):
    """
        封装paramiko类，实现ssh执行命令、上传下载文件
    """

    def __init__(self, hostip, username, ssh_key="", password="", port=22, timeout=5, encoding="utf8", **kwargs):
        """
        :param hostip: 主机IP
        :param username: 主机用户
        :param password:  主机密码
        :param port: 主机端口，默认22，可传
        """
        self.hostip = hostip
        self.port = port
        self.username = username.strip()
        self.password = password.strip()
        self.ssh_key = ssh_key
        self.timeout = timeout
        self.encoding = encoding
        ips = []
        try:
            addr_info = socket.getaddrinfo(socket.gethostname(), None)
            for addr in addr_info:
                ips.append(addr[4][0])
        except socket.gaierror:
            pass

        self.is_local = True if self.hostip in ips else False

        # 创建一个ssh对象，用于ssh登录以及执行操作
        self.obj = paramiko.SSHClient()
        # 如果第一次连接陌生的IP,自动选择yes确认连接
        self.obj.set_missing_host_key_policy(paramiko.AutoAddPolicy())
        # 连接服务器
        if not self.is_local:
            try:
                if self.password:
                    self.obj.connect(hostname=self.hostip, port=self.port, username=self.username,
                                     password=self.password,
                                     timeout=self.timeout)
                    self.objsftp = self.obj.open_sftp()
                    self.chan = self.obj.invoke_shell()
                    self.chan.settimeout(9000)

                elif self.ssh_key:
                    self.pkey = paramiko.RSAKey.from_private_key_file(self.ssh_key)
                    self.obj.connect(hostname=self.hostip, port=self.port, username=self.username, pkey=self.pkey,
                                     timeout=self.timeout)
                    self.objsftp = self.obj.open_sftp()
                    self.chan = self.obj.invoke_shell()
                    self.chan.settimeout(9000)
                else:
                    raise SSHAuthInfoMissed()
            except AuthenticationException:
                raise SSHAuthFailed()
            except NoValidConnectionsError:
                raise SSHPortFailed()
            except socket.timeout:
                raise SSHConnectFailed()
            except socket.gaierror:
                raise SSHPortOrIpIllegal()

    def run_cmd(self, cmd):
        """
        执行命令
        :param cmd: 需要执行的命令
        :return: 执行结果
        """
        cmd = monkey_sudo(self.password, cmd)
        if self.is_local:
            code, res = subprocess.getstatusoutput(cmd)
            err = res
        else:
            stdin, stdout, stderr = self.obj.exec_command(cmd, get_pty=True)
            code, res, err = stdout.channel.recv_exit_status(), stdout.read(), stderr.read()
            r = re.compile("\[sudo.*: ")
            res = r.sub("", to_str(res))
        # result = res if res else err
        res = {
            "code": code,
            "stdout": to_str(res),
            "stderr": to_str(err)
        }
        return res

    def run_cmdlist(self, cmdlist):
        self.resultList = []
        for cmd in cmdlist:
            # stdin, stdout, stderr = self.obj.exec_command(cmd)
            # self.resultList.append(stdout.read())
            res = self.run_cmd(cmd)
            self.resultList.append(res)
        return self.resultList

    def get(self, remotepath, localpath):
        """
        从远程服务器获取文件到本地
        """
        if self.is_local:
            localpath_dir = os.path.dirname(localpath)
            if not os.path.exists(localpath_dir):
                os.makedirs(localpath_dir)
            try:
                shutil.copy(remotepath, localpath)
            except IOError as e:
                print("Unable to copy file. %s" % e)
                exit(2)
            except:
                print("Unexpected error:", sys.exc_info())
                exit(2)
        else:
            self.objsftp.get(remotepath, localpath)

    def _file_translate(self, tmp_path, remote_path):
        # 去掉.tmp后缀，获取文件路径
        # ok_tmp_path = os.path.splitext(tmp_path)[0]
        result = self.run_cmd("mv {} {}".format(tmp_path, remote_path))
        result = self.run_cmd("chown root.root -R {}".format(remote_path))
        return result

    def put(self, localpath, remotepath, ignore=''):
        """
        从本地推送到远程服务器
        如果本地是目录，远程也会是目录
        如果本地是文件，远程也会是文件
        """
        # 判断本地要传的是文件还是目录，如果是文件直接传即可，目录需要打包
        if self.is_local:
            if os.path.isdir(localpath):
                try:
                    shutil.copytree(localpath, remotepath)
                except FileExistsError:
                    shutil.rmtree(remotepath, ignore_errors=True)
                    shutil.copytree(localpath, remotepath)
            else:
                try:
                    shutil.copy(localpath, remotepath)
                except FileNotFoundError:
                    os.makedirs(os.path.dirname(remotepath))
                    shutil.copy(localpath, remotepath)
                except shutil.SameFileError:
                    pass

        else:
            try:
                # 远程上传目录逻辑
                if os.path.isdir(localpath):
                    # 去掉path末尾斜杠
                    if localpath[-1] == '/':
                        localpath = localpath[0:-1]

                    # 打包tar文件，临时压缩文件，传输完成会删掉
                    # tar包名称
                    tar_name = "{}-{}.tar.gz".format(localpath.split("/")[-1], int(time.time()))
                    # 去掉远程最后一层目录,获取上层目录(创建目录)，因为后续还需要解压
                    top_remote_path = os.path.dirname(remotepath)
                    # 检查是否存在上层目录，不存在即创建
                    try:
                        self.objsftp.stat(top_remote_path)
                    except IOError:
                        result = self.run_cmd("mkdir -pv {}".format(top_remote_path))
                    # tar包路径
                    remote_tarfile_path = os.path.join(top_remote_path, tar_name)
                    # 把目录归档压缩成tar包
                    with tarfile.open(tar_name, "w:gz") as tar:
                        def filter_func(tarinfo):
                            if tarinfo.name == os.path.join(os.path.basename(localpath),ignore):
                                return None
                            else:
                                return tarinfo
                        tar.add(localpath, arcname=os.path.basename(localpath), filter=filter_func)
                    # 定义远程临时path（先放tmp中）
                    remote_tarfile_path_tmp = os.path.join("/tmp", "{}.tmp".format(tar_name))
                    self.objsftp.put(tar_name, remote_tarfile_path_tmp)
                    # 移动到正确的远程目录
                    # self._file_translate(
                    #     tmp_path=remote_tarfile_path_tmp, remote_path=top_remote_path
                    # )
                    # 远端进行解压
                    tar_command = "tar -zxf {} -C {}".format(remote_tarfile_path_tmp, top_remote_path)
                    result = self.run_cmd(tar_command)
                    if result["code"] != 0:
                        raise Exception(result)
                    # 删除本地临时tar文件
                    os.remove(tar_name)
                    # 删除远端临时tar文件
                    result = self.run_cmd("rm -f {}".format(remote_tarfile_path_tmp))
                    if result["code"] != 0:
                        raise Exception(to_str(result))

                elif os.path.isfile(localpath):
                    remote_dir_name = os.path.dirname(remotepath)
                    try:
                        self.objsftp.stat(remote_dir_name)
                    except IOError:
                        result = self.run_cmd("mkdir -pv {}".format(remote_dir_name))
                    # 定义远程临时path（先放tmp中）
                    file_name = "{}.tmp".format(os.path.basename(remotepath))
                    remote_path_tmp = os.path.join("/tmp", file_name)
                    self.objsftp.put(localpath, remote_path_tmp)
                    self._file_translate(
                        tmp_path=remote_path_tmp, remote_path=remotepath
                    )
                else:
                    print(FileTypeErrors().WRONG_FILE_TYPE)
            except Exception as e:
                raise Exception(e)

    def close(self):
        if not self.is_local:
            self.objsftp.close()
            self.obj.close()
        else:
            pass


if __name__ == '__main__':
    pass



from __future__ import absolute_import
from __future__ import unicode_literals
import os
from redis.sentinel import Sentinel
from laipvt.model.cmd import DockerImageModel
from laipvt.model.harbor import HarborModel
from laipvt.handler.confighandler import CheckResultHandler, ServerHandler
from laipvt.handler.middlewarehandler import HarborConfigHandler
from laipvt.sysutil.template import FileTemplate
from laipvt.sysutil.util import path_join, log, ssh_obj, to_object, walk_sql_path
from laipvt.handler.middlewarehandler import MiddlewareConfigHandler
from laipvt.model.cmd import ComposeModel
from laipvt.model.sql import SqlModule
from laipvt.model.server import runcmd
from laipvt.sysutil.command import CREATE_NS, ISTIO_INJECTION_NS, KUBECTL_APPLY, MKDIR_DIR, CHMOD_777, \
    HELM_LIST, HELM_INSTALL, CREATE_DB, INIT_IDENTITY_USER, HELM_INSTALL_TF_SERVICE, HELM_UPGRADE_INSTALL


class ServiceInterface:
    def __init__(self, check_result: CheckResultHandler, service_path):
        """
        check_result: 对象，前置检查结果
        service_path: 对象，服务进程详情
        """
        self.check_result = check_result
        self.service_path = service_path

        self.middleware_servers = self.check_result.servers
        self.middleware_server_list = self.middleware_servers.get_role_ip("master")

        self.middleware_cfg = to_object(MiddlewareConfigHandler("mysql").get_all_config_with_check_result())
        for k, v in self.middleware_cfg.items():
            if not self.middleware_cfg[k]["ipaddress"]:
                self.middleware_cfg[k]["ipaddress"] = self.middleware_server_list
        self.middleware_cfg.update(self.check_result.__dict__)

        self.servers = check_result.servers.get()
        self.service_info = service_path.config
        self.private_deploy_version = self.service_info.tag

        self.templates_dir = self.service_path.templates
        self.data_dir = self.service_path.data
        self.deploy_dir = self.check_result.deploy_dir

        self.service_charts_remote = path_join(self.deploy_dir, "charts")
        self.harbor_cfg = HarborConfigHandler().get_config_with_check_result()
        try:
            harbor_ip = self.harbor_cfg["harbor"]["lb"]
        except IndexError:
            harbor_ip = self.check_result.servers.get_role_ip("harbor")[0]
        self.registry_hub = "{}:{}".format(harbor_ip, self.harbor_cfg["harbor"]["nginx_harbor_proxy_port"])

        self.etcd_servers = self.check_result.servers.get_role_ip("master")
        self.etcd_endpoint = "\,".join(
            ["{}:{}".format(server, self.middleware_cfg.etcd.http_port) for server in self.etcd_servers]
        )

        self.env_k8s_config_src = path_join(self.templates_dir, "env_k8s_config")
        self.env_k8s_config_dest = path_join(self.templates_dir, "env_k8s_config_dest")
        self.env_k8s_config_remote = path_join(self.deploy_dir, "env_k8s_config_dest")

        self.nginx_compose_file = path_join(self.deploy_dir, "nginx", "docker-compose.yml")

        self.servers = self.check_result.servers.get()
        self.middleware_cfg["k8s_hosts"] = [x.ipaddress for x in self.servers]
        self.master_hosts = self.check_result.servers.get_role_obj("master")
        self.middleware_cfg["k8s_masters"] = [x.ipaddress for x in self.master_hosts]

        self.replicas = 1
        self.nodes = self.check_result.servers.get_role_obj("node")
        self.master_host = self.check_result.servers.get_role_obj("master")[0]
        self.harbor_hosts = self.check_result.servers.get_role_obj("harbor")

        self.rabbitmq_init_file_template_path = path_join(self.templates_dir, "init_rabbitmq.tmpl")
        self.rabbitmq_init_file_path = path_join(self.templates_dir, "init_rabbitmq.sh")

        self.redis_init_file_template_path = path_join(self.templates_dir, "init_redis.tmpl")
        self.redis_init_file_path = path_join(self.templates_dir, "init_redis.sh")

    def push_images(self, project):
        harbor = HarborModel(username="admin", host=self.registry_hub, password=self.middleware_cfg["harbor"]["password"])
        if project not in harbor.list_project():
            harbor.create_project(project)
        for image in os.listdir(self.service_path.images):
            image_path = path_join(self.service_path.images, image)
            log.info("将镜像push到私有仓库: {}".format(image_path))
            # print(image_path)
            docker = DockerImageModel(image=image_path, project=project, repo=self.registry_hub)
            docker.run()

    def _send_file(self, src, dest, role=""):
        l = []
        if role:
            for server in self.servers:
                if server.role.check(role):
                    l.append(server)
        else:
            l = self.servers
        for server in l:
            log.info("分发{}到{}:{}".format(src, server.ipaddress, dest))
            ssh_cli = ssh_obj(ip=server.ipaddress, user=server.username, password=server.password, port=server.port)
            try:
                ssh_cli.put(src, dest)
            except Exception as e:
                log.error(e)
                exit(2)
            finally:
                ssh_cli.close()

    def _exec_command_to_host(self, cmd, server: ServerHandler, check_res=True) -> dict:
        log.info("主机 {} 执行命令: {}".format(server.ipaddress, cmd))
        if isinstance(cmd, list):
            ssh_cli = ssh_obj(ip=server.ipaddress, user=server.username, password=server.password, port=server.port)
            res_list = ssh_cli.run_cmdlist(cmd)
            ssh_cli.close()
            if check_res:
                for res in res_list:
                    if res["code"] != 0:
                        log.error("{} {}".format(res["stdout"], res["stderr"]))
                        exit(2)
            return res_list
        if isinstance(cmd, str):
            ssh_cli = ssh_obj(ip=server.ipaddress, user=server.username, password=server.password, port=server.port)
            res = ssh_cli.run_cmd(cmd)
            ssh_cli.close()
            if check_res:
                if res["code"] != 0:
                    log.error("{} {}".format(res["stdout"], res["stderr"]))
                    exit(2)
            return res
        else:
            log.error("{}传入命令格式存在错误".format(cmd))
            exit(2)

    def _create_namespace(self, namespaces, istio_injection_namespaces=""):
        if namespaces:
            for ns in namespaces:
                log.info("创建namespace: {}".format(ns))
                cmd = CREATE_NS.format(ns)
                self._exec_command_to_host(cmd=cmd, server=self.master_host, check_res=False)
        if istio_injection_namespaces:
            for ns in istio_injection_namespaces:
                inject_cmd = ISTIO_INJECTION_NS.format(ns)
                self._exec_command_to_host(cmd=inject_cmd, server=self.master_host, check_res=False)

    def deploy_istio(self):
        # 渲染istio配置
        FileTemplate(
            self.middleware_cfg,
            path_join(self.env_k8s_config_src, "istio"),
            path_join(self.env_k8s_config_dest, "istio")
        ).fill()
        self._send_file(src=self.env_k8s_config_dest, dest=self.env_k8s_config_remote)
        cmd = KUBECTL_APPLY.format(path_join(self.env_k8s_config_remote, "istio"))
        self._exec_command_to_host(cmd=cmd, server=self.master_host)

    def _create_logs_dir(self, project):
        log_path = os.path.join(self.deploy_dir, "Logs", project)
        log.info("创建服务日志目录:{}".format(log_path))
        cmd = [
            MKDIR_DIR.format(log_path),
            CHMOD_777.format(log_path)
        ]
        for server in self.servers:
            self._exec_command_to_host(cmd=cmd, server=server)

    def start_service(self, project, version):
        self._send_file(src=self.service_path.charts, dest=self.service_charts_remote)
        for service, processes in self.service_path.config.services.items():
            for process in processes:
                log.info("{}开始部署".format(process))

                check_cmd = HELM_LIST.format(
                    process, process
                )
                check_results = self._exec_command_to_host(cmd=check_cmd, server=self.master_host, check_res=False)
                if check_results["code"] == 0:
                    log.warning("{} helm部署记录中已经存在，不做更新，如需要更新，可以先行删除".format(process))

                else:
                    self._create_logs_dir(service)
                    file_path = os.path.join(self.service_charts_remote, process)
                    # print(file_path)
                    cmd = HELM_INSTALL.format(
                        process=process, replicas=self.replicas,
                        registry_hub=path_join(self.registry_hub, project),
                        image_name=process, image_tag=version,
                        pvt_work_dir=self.deploy_dir,
                        etcd_endpoint=self.etcd_endpoint,
                        file_path=file_path)

                    self._exec_command_to_host(cmd=cmd, server=self.master_host)

    def upgrade_service(self, project):
        for service, processes in self.service_path.config.services.items():
            for process in processes:
                log.info("{}开始部署".format(process))
                self._create_logs_dir(service)
                file_path = os.path.join(self.service_path.charts, process)
                # print(file_path)
                cmd = HELM_UPGRADE_INSTALL.format(
                    process=process, replicas=self.replicas,
                    registry_hub=path_join(self.registry_hub, project),
                    image_name=process, image_tag=self.private_deploy_version,
                    pvt_work_dir=self.deploy_dir,
                    etcd_endpoint=self.etcd_endpoint,
                    file_path=file_path)
                self._exec_command_to_host(cmd=cmd, server=self.master_host)

    def proxy_on_nginx(self, nginx_template, nginx_tmp, nginx_file_remote):
        log.info("渲染nginx代理配置文件: {} -- > {}".format(nginx_template, nginx_tmp))
        FileTemplate(self.middleware_cfg, nginx_template, nginx_tmp).fill()
        self._send_file(src=nginx_tmp, dest=nginx_file_remote)
        compose_cmd = ComposeModel(self.nginx_compose_file)
        for server in self.servers:
            self._exec_command_to_host(cmd=compose_cmd.restart(), server=server, check_res=True)

    def init_mysql(self, sql_path):
        log.info(sql_path)
        db_info = walk_sql_path(sql_path)
        if self.middleware_cfg.mysql.is_deploy:
            mysql_host = self.master_host
        else:
            mysql_host = self.middleware_cfg.mysql.ipaddress[0]
        sql = SqlModule(host=mysql_host, port=int(self.middleware_cfg.mysql.port),
                        user=self.middleware_cfg.mysql.username, passwd=self.middleware_cfg.mysql.password)
        for db_name, sql_files in db_info.items():
            create_db = CREATE_DB.format(db_name=db_name)
            sql.insert_sql(create_db)
            sql.use_db(db_name)
            for sql_file in sql_files:
                sql.import_from_file(sql_file, file_eof=";\n")

    def init_rabbitmq(self):
        log.info("渲染初始化RabbitMQ脚本: {} -- > {}".format(
            self.rabbitmq_init_file_template_path,
            self.rabbitmq_init_file_path)
        )
        FileTemplate(self.middleware_cfg, self.rabbitmq_init_file_template_path, self.rabbitmq_init_file_path).fill()

        fp = open(self.rabbitmq_init_file_path)
        log.info("开始执行初始化RabbitMQ命令")

        for cmd in fp.readlines():
            if cmd.strip():
                log.info(cmd)
                code, res = runcmd(cmd)
                if code != 0:
                    log.error(res)
                    exit(2)

    def init_redis(self):
        if self.middleware_cfg["redis"]["is_deploy"]:
            # 如果是自建redis服务，连接哨兵服务器，获取master节点地址
            pool = []
            for host in self.middleware_servers.get_role_obj("master"):
                redis_endpoint = (host.ipaddress, self.middleware_cfg["redis"]["port_sentinel"])
                pool.append(redis_endpoint)
            sentinel = Sentinel(pool, socket_timeout=0.5)
            # 获取主服务器地址
            master_address = sentinel.discover_master(self.middleware_cfg.redis.master_name)[0]
            # server_object_list = self.check_result.servers.search_server(key="ipaddress", value=master)
            self.middleware_cfg["redis"]["master_address"] = master_address

        log.info("渲染初始化redis脚本: {} -- > {}".format(
            self.redis_init_file_template_path,
            self.redis_init_file_path)
        )
        FileTemplate(self.middleware_cfg, self.redis_init_file_template_path, self.redis_init_file_path).fill()

        fp = open(self.redis_init_file_path)
        log.info("开始执行初始化redis命令")

        for cmd in fp.readlines():
            if cmd.strip():
                log.info(cmd)
                code, res = runcmd(cmd)
                if code != 0:
                    log.error(res)
                    exit(2)

    def init_identity_user(self):
        init_user_cmd = INIT_IDENTITY_USER
        self._exec_command_to_host(cmd=init_user_cmd, server=self.servers[0], check_res=False)

    def deploy_tf_service(self, module_name, tf_image_name):
        # self._send_file(src=self.service_path.charts, dest=self.service_charts_remote)
        log.info("{}开始部署".format(module_name))
        check_cmd = HELM_LIST.format(
            module_name, module_name
        )
        check_results = self._exec_command_to_host(cmd=check_cmd, server=self.master_host, check_res=False)
        if check_results["code"] == 0:
            log.warning("{} helm部署记录中已经存在，不做更新，如需要更新，可以先行删除".format(module_name))

        else:
            file_path = os.path.join(self.service_charts_remote, module_name)
            # print(file_path)
            tag = "2.4.0-cpu" if self.service_path.config.machine_type == "cpu" else "2.4.0-gpu"
            cmd = HELM_INSTALL_TF_SERVICE.format(
                process=module_name, replicas=self.replicas,
                registry_hub=path_join(self.registry_hub, "middleware"),
                image_name=tf_image_name, image_tag=tag,
                pvt_work_dir=self.deploy_dir,
                etcd_endpoint=self.etcd_endpoint,
                file_path=file_path)

            self._exec_command_to_host(cmd=cmd, server=self.master_host)



from __future__ import absolute_import
from __future__ import unicode_literals

import pymongo
import requests
from laipvt.interface.middlewareinterface import MiddlewareInterface
from laipvt.handler.middlewarehandler import SiberConfigHandler
from laipvt.handler.confighandler import CheckResultHandler
from laipvt.sysutil.template import FileTemplate
from laipvt.sysutil.util import path_join, ssh_obj, log, status_me
from laipvt.model.cmd import ComposeModel
from laipvt.sysutil.siber_main import Mage_test

class SiberController(MiddlewareInterface):
    def __init__(self, result: CheckResultHandler, handler: SiberConfigHandler, template: str):
        super(SiberController, self).__init__(result, handler, template)
        self.conf_tmp = path_join("/tmp")
        self.conf_template = path_join(self.template)
        self.conf_file = path_join(self.base_dir, "config")
        self.list_conf = ("siber.conf", "siber-web.conf", "siber-nginx.conf")
        self.mongo_data_source = path_join(self.template, "siber_mongo")
        self.mongo_data_remote = path_join(self.base_dir, "siber_mongo")
        self.siber_cfg = SiberConfigHandler().get_all_config_with_check_result()
        self.siber_cfg["siber"] = SiberConfigHandler().get_config_with_check_result()["siber"]
        self.siber_cfg["siber"]["ipaddress"] = self.handler.cfg["ipaddress"]
        self.siber_host = self.master_server[0].ipaddress
        self.tagid_list = []
        self.casid_list = []

    def _generic_config(self):
        log.info("渲染 Siber 配置文件")
        for num_id in range(len(self.list_conf)):
            dest = path_join(self.conf_template, self.list_conf[num_id])
            src_tmp = path_join(self.conf_tmp, self.list_conf[num_id])
            src = path_join(self.conf_file, self.list_conf[num_id])
            FileTemplate(self.siber_cfg, dest, src_tmp).fill()
            self.send_config_file(self.master_server[0], src_tmp, src)
        self.send_config_file(self.master_server[0], self.mongo_data_source, self.mongo_data_remote)
        self.generate_docker_compose_file(self.siber_cfg)
        self._send_docker_compose_file(self.master_server[0])

    def _send_docker_compose_file(self, host):
        log.info("分发%s docker-compose配置文件至 %s" % (self.middleware_name, host.ipaddress))
        ssh_cli = ssh_obj(ip=host.ipaddress, user=host.username, password=host.password, port=host.port)
        try:
            ssh_cli.put(self.docker_compose_file_tmp, self.docker_compose_file)
        except Exception as e:
            log.error(e)
            exit(2)
        finally:
            ssh_cli.close()

    def _init_siber(self):
        super().wait_for_service_start()
        log.info("初始化 siber-mongo 数据")
        try:
            init_mongo_cmd = "docker exec siber-mongo mongorestore -u admin -p admin -d admin siber_mongo/siber"
            server = self.master_server[0]
            ssh_cli = ssh_obj(ip=server.ipaddress, user=server.username, password=server.password, port=server.port)
            res = ssh_cli.run_cmd(init_mongo_cmd)
            if res["code"] != 0:
                log.error("初始化 siber-mongo 数据失败:{}".format(res["stderr"]))
            else:
                log.info("初始化 siber-mongo 数据成功")
        except Exception as e:
            log.error(e)
            log.error("初始化mongo数据失败")

    def _start(self):
        compose_cmd = ComposeModel(self.docker_compose_file)
        for server in self.master_server[:1]:
            log.info("启动 %s: %s服务" % (server.ipaddress, self.middleware_name))
            ssh_cli = ssh_obj(ip=server.ipaddress, user=server.username, password=server.password, port=server.port)
            try:
                res = ssh_cli.run_cmd(compose_cmd.up())
                if res["code"] == 0:
                    log.info("启动 %s %s 服务成功" % (server.ipaddress, self.middleware_name))
                else:
                    log.error("启动 %s %s 服务失败: %s " % (server.ipaddress, self.middleware_name, res["stdout"]))
            except Exception as e:
                log.error(e)
            finally:
                ssh_cli.close()

    def _check(self):
        super().wait_for_service_start()
        for server in self.master_server[:1]:
            res = self.check_port(server.ipaddress, self.siber_cfg["siber"]["port"])
            if res:
                log.info("机器:{} Siber-nginx代理端口{}访问正常".format(server.ipaddress, self.siber_cfg["siber"]["port"]))
            else:
                log.error("机器:{} Siber-nginx代理端口{}访问异常".format(server.ipaddress, self.siber_cfg["siber"]["port"]))

    def send_request(self, url, data):
        resp = requests.post(url=url, json=data)
        result = resp.json()
        return result

    def run_plan(self, plan_id):
        data = {
            "plan_info": {
                "plan_id": plan_id
            },
            "trigger_condition": {
                "environment_name": "prod"
            }
        }
        url = "http://{}:{}/siberhttp/run/plan".format(self.siber_host, self.siber_cfg["siber"]["port"])

        return self.send_request(url, data)

    def update_planversion(self, plan_version):
        data = {
        "manage_mode":"UPDATE",
            "plan_info": {
                "plan_name": "私有部署定制化mage plan",
                "interface_type": "http",
                "environment_name": "测试环境",
                "environment_id": "5e940f347dcd1000016d8378",
                "threads": 0,
                "flow_list": [
                    "5f1926a438cb970001f5a936"
                ],
                "plan_id": "5f1926b438cb970001f5a93d",
                "version_control": plan_version
            }
        }
        url = "http://{}:{}/siberhttp/manage/plan".format(self.siber_host, self.siber_cfg["siber"]["port"])
        return self.send_request(url, data)

    def replace_mage_collection_tag(self, tag_id):
        '''
        根据tag列表到mongo中取出tagid
        根据tagid将case取出
        将case到如到指定的flow中
        '''
        try:
            self.mongo_client = pymongo.MongoClient("mongodb://{}:27027/".format(self.siber_host))
            self.my_db = self.mongo_client.admin
            self.auth = self.my_db.authenticate("admin", "admin", mechanism='SCRAM-SHA-1')

            for tag_id in self.my_db.collection_tag.find({"tagname": {"$in": tag_id}}):
                self.tagid_list.append(tag_id["tagid"])
            for cas_tags in self.tagid_list:
                for res in self.my_db.collection_case.find({"$and": [{"casetags": cas_tags}, {"invaliddate": 0}]}):
                    if "BASIC" not in res["casename"]:
                        self.casid_list.append(res["_id"])
            self.my_db.collection_flow.update({"flowid": "5f1926a438cb970001f5a936"}, {"$set": {"caselist": self.casid_list}})
            self.run_mage_siber_test()
        except Exception as e:
            log.error(e)
            exit(2)

    def run_mage_siber_test(self):
        try:
            log.info("替换 Siber env,oss 配置")
            Mage_test(self.siber_host).replace_private_oss_url()
            Mage_test(self.siber_host).update_private_mage_env_info()
            self.update_planversion("v1.10")
            if self.run_plan("5f1926b438cb970001f5a93d"):
                log.info("请检查集成测试平台 '私有部署定制化mage plan' 运行结果")
                log.info("集成测试平台访问地址http://{}:88".format(self.siber_host))
            else:
                log.error("集成测试平台运行失败")
        except Exception as e:
            log.error(e)
            exit(2)

    @status_me("middleware")
    def deploy_siber(self):
        self._generic_config()
        self._start()
        self._init_siber()
        self._check()



from __future__ import absolute_import
from __future__ import unicode_literals

from laipvt.sysutil.conf import YamlConfig
from laipvt.sysutil.gvalue import PORT_MAP

module_require_tfserver = ("ocr_standard", "captcha", "nlp")
tfserver_name = {
    "ocr_standard": ["ocr-ctpn-tf-server", "ocr-text-recognition-tf-server",
                     "ocr-unet-table-tf-serving", "semantic-correct"],
    "captcha": ["verification-tf-serving"],
    "nlp": ["bert-service-tf"]
}

tfserver_image_name = {
    "ocr_standard": "tfserving",
    "captcha": "tfserving",
    "nlp": "tfserving"
}

service_module_relation = {
    "commander": [x for x in range(0, 11)],
    "mage": [x for x in range(31, 51)],
    "wulai": [x for x in range(11, 31)],
    "nlp": [33, 34, 35],
    "captcha": [32],
    "ocr_standard": [36, 37],
    "ocr": [x for x in range(51, 129)]
}

machine_module_relation = {
    "cpu": [x for x in range(34, 129) if x % 2 == 0],
    "gpu": [x for x in range(34, 129) if x % 2 != 0]
}
ocr_standard = range(36, 51)
ocr = range(51, 129)
module_info = {
    0: ("commander", "UiBot Commander"),
    11: ("wulai", "吾来对话机器人"),
    31: ("mage", "UiBot Mage"),
    32: ("captcha", "验证码识别"),
    33: ("geo", "标准地址"),
    34: ("nlp", "文本分类、信息抽取"),
    35: ("nlp", "文本分类、信息抽取(GPU)"),
    36: ("ocr_document_standard", "通用文字识别(标准版 CPU)"),
    37: ("ocr_document_standard_gpu", "通用文字识别(标准版 GPU)"),
    51: ("ocr_document_gpu", "通用文字识别(高级版 GPU)"),
    52: ("ocr_document", "通用文字识别(高级版 CPU)"),
    53: ("ocr_table_gpu", "通用表格识别(高级版 GPU)"),
    54: ("ocr_table", "通用表格识别(高级版 CPU)"),
    55: ("ocr_receipt_gpu", "通用票据识别(高级版 GPU)"),
    56: ("ocr_receipt", "通用票据识别(高级版 CPU)"),
    57: ("ocr_idcard_gpu", "通用卡证识别-身份证(高级版 GPU)"),
    58: ("ocr_idcard", "通用卡证识别-身份证(高级版 CPU)"),
    59: ("ocr_bankcard_gpu", "通用卡证识别-银行卡(高级版 GPU)"),
    60: ("ocr_bankcard", "通用卡证识别-银行卡(高级版 CPU)"),
    61: ("ocr_vehicle_gpu", "通用卡证识别-机动车登记证(高级版 GPU)"),
    62: ("ocr_vehicle", "通用卡证识别-机动车登记证(高级版 CPU)"),
    63: ("ocr_vehiclelicense_gpu", "通用卡证识别-机动车行驶证(高级版 GPU)"),
    64: ("ocr_vehiclelicense", "通用卡证识别-机动车行驶证(高级版 CPU)"),
    65: ("ocr_biz_gpu", "通用卡证识别-营业执照(高级版 GPU)"),
    66: ("ocr_biz", "通用卡证识别-营业执照(高级版 CPU)")
}

menu_relation = {
    1001: {
        36: 100101,
        37: 100102,
        51: 100103,
        52: 100104
    }
}

middleware_port_relation = {
    "minio": {
        "port": 9000,
        "nginx_proxy_port": 10000
    },
    "harbor": {
        "http_port": 8888,
        "nginx_harbor_proxy_port": 8889
    },
    "redis": {
        "port": 6379,
        "port_sentinel": 26379
    },
    "nginx": {
        "commander_port": 8080,
        "commander_tenant_port": 8081,
        "k8s_proxy_port": 6444,
        "mage_proxy_port": 8082
    },
    "mysql": {
        "port": 3306,
        "proxysql_cluster_port": 6032,
        "proxysql_port": 6033,
        "nginx_proxy_port": 6034
    },
    "elasticsearch": {
        "http_port": 9200,
        "tcp_port": 9300
    },
    "rabbitmq": {
        "port": 5672,
        "manage_port": 15672,
        "empd_port": 4369,
        "erlang_port": 25672
    },
    "etcd": {
        "http_port": 12379,
        "tcp_port": 12380
    },
    "identity": {
        "port": 6060,
        "nginx_proxy_port": 6061
    },
    "monitor": {
        "grafana_port": 3000,
        "prometheus_port": 9090,
        "mysql_exporter_port": 9104,
        "redis_exporter_port": 9121,
        "rabbitmq_exporter_port": 9419,
        "elasticsearch_exporter_port": 9114,
        "node_exporter_port": 9100,
        "k8s_metrics_port": 31388,
        "istio_prometheus_port": 31390
    },
    "siber": {
        "port": 88
    },
    "ocr": {
        "ocr_document_gpu": 30006,
        "ocr_table_gpu": 30007,
        "ocr_receipt_gpu": 30008,
        "ocr_idcard_gpu": 30009,
        "ocr_bankcard_gpu": 30013,
        "ocr_vehicle_gpu": 30010,
        "ocr_vehiclelicense_gpu": 30011,
        "ocr_biz_gpu": 30012,
        "ocr_passport_gpu": 30014
    }
}

def init_port_config():
    conf = YamlConfig(PORT_MAP, data=middleware_port_relation)
    conf.write_file(backup=False)

def find_module_by_key(module: str) -> list:
    l = []
    for id in module_info:
        if module_info[id][0] == module:
            res = {}
            res['id'] = id
            res['module'] = module
            res['description'] = module_info[id][1]
            l.append(res)
    return l

def get_module_description(module_id: int) -> str:
    return module_info[module_id][1]

def get_module_key(module_id: int) -> str:
    return module_info[module_id][0]

def get_module_keys(module_ids: list) -> list:
    return [get_module_key(x) for x in module_ids]

def get_all_ports(middleware="") -> list:
    ''':return list[str]'''
    l = []
    try:
        relation = YamlConfig(PORT_MAP).read_file()
    except Exception:
        relation = middleware_port_relation
    if middleware:
        try:
            for k in relation[middleware]:
                l.append(str(relation[middleware][k]))
        except KeyError:
            pass
    else:
        for mid in relation:
            for k in relation[mid]:
                l.append(str(relation[mid][k]))
    return l

def get_port(middleware: str, key: str) -> int:
    try:
        relation = YamlConfig(PORT_MAP).read_file()
    except Exception:
        relation = middleware_port_relation

    try:
        return relation[middleware][key]
    except KeyError:
        for k in relation[middleware].keys():
            if key in k:
                return relation[middleware][k]
        return 0